/* ===================================================================================================
* WARNING – This file is part of the base implementation for WebMaker, so it should not be edited or changed for any project.
* These files are replaced if a project is re-imported to the WebMaker Studio or migrated to a new version of the product.
* For guidance on 'How do I override or clone Hyfinity webapp files such as CSS & javascript?', please read the following relevant FAQ entry:
* http://www.hyfinity.net/faq/index/solution_id/1113
==================================================================================================== */

/*
 * DisplayUitls.js
 *
 * Utility functions for altering the display of HTML componenets from javascript
 *
 * Company: Hyfinity Ltd
 * Copyright (c) 2003
 */


if (typeof(dojo) != 'undefined')
{
    dojo.require("dojo.parser");
    dojo.require("dojo.fx");
    dojo.require("dojo.on");
}


/**
 * Root object to contain all hyfinity functions.
 * This should remove the chance of a name conflict with other functionality.
 * @private
 *
 */
var hyf =
{
    version: '2.0',
    pageLoaded: false,
    afterLoadOnLoadFunctions: {}
};

/**
 * Attaches the specified function to be called when the specified event occurs on the given object.
 * This removes the need to worry about browser complexities from the main code.
 * For now we delegate to dojo to handle this for us if possible.
 *
 * @param object {object) The object to attach the event to
 * @param event {string} Specifiying the actual event that the fucntion should be called for e.g. 'onclick'
 * @param func {function} The function that should be called when this event occurs.
 * @author Hyfinity Limited
 */
hyf.attachEventHandler = function(object, event, func)
{
    if (event.indexOf('on') != 0)
        event = 'on' + event;

    if ((object == window) && (event.indexOf('onload') == 0) && (hyf.pageLoaded))
    {
        if (event.indexOf('-') != -1)
        {
            var identifier = event.substr(event.indexOf('-') + 1);

            if (!hyf.afterLoadOnLoadFunctions[identifier])
                hyf.afterLoadOnLoadFunctions[identifier] = new Array();

            hyf.afterLoadOnLoadFunctions[identifier].push(func)
        }
        else
            func();

        return;
    }
    if (event.indexOf('onload') == 0) event = 'onload';

    //use dojo to attach the event if possible
    if ((typeof(dojo) != 'undefined') && (dojo.on))
    {
        if ((object == window) && (event == 'onload'))
            dojo.addOnLoad(func)
        else
            dojo.on(object, event.substring(2), func);
    }
    else if (typeof(object.attachEvent) != 'undefined') //IE format
        object.attachEvent(event, func);
    else if (typeof(object.addEventListener) != 'undefined') //W3C format
        object.addEventListener(event.substring(2), func, false);
};

hyf.attachEventHandler(window, 'onload', function() {hyf.pageLoaded = true;});

/**
 * Fires the given event on the provided object.
 * @param object {object) The object to fire the event against.
 * @param event {string} the name of the event to fire. Should not have the 'on' prefix e.g. 'click'
 * @author Hyfinity Limited
 */
hyf.fireEvent = function(object, event)
{
    if (event.indexOf('on') == 0)
        event = event.substring(2);

    if ((typeof(dojo) != 'undefined') && (dojo.on))
    {
        return dojo.on.emit(object, event, {bubbles: true, cancelable: true});
    }
    else if (document.createEventObject)
    {
        // dispatch for IE
        var evt = document.createEventObject();
        return object.fireEvent('on' + event, evt)
    }
    else
    {
        // dispatch for firefox + others
        var evt = document.createEvent("HTMLEvents");
        evt.initEvent(event, true, true ); // event type,bubbling,cancelable
        return !object.dispatchEvent(evt);
    }
}


/*
 * Object containing all Hyfinity utility functions
 */
hyf.util = {};


/**
 * Generic function for toggling the display of a component.
 * @param component {Group_Name | Field_Name} the field, group or HTML component (or its ID) to show/hide
 * @param method {string} (optional) An optional string specifying the required visibility of the
 *               component ('show' or 'hide'). A Null value will toggle from show/hide as appropriate.
 * @param animate {boolean} (optional) if true the show/hide will be performed gradually.
 * @author Hyfinity Limited
 */
hyf.util.toggleComponent = function(component, method, animate)
{
    if (typeof(component) == 'string')
        component = document.getElementById(component);

    if ((component != null) && (typeof(component) != 'undefined'))
    {
        if ((method == null) || (typeof(method) == 'undefined'))
        {
            if (hyf.util.getCurrentStyle(component, 'display') == 'none')
            {
                hyf.util.showComponent(component, animate);
            }
            else
            {
                hyf.util.hideComponent(component, animate);
            }
        }
        else
        {
            if (method == 'hide')
            {
                hyf.util.hideComponent(component, animate);
            }
            else
            {
                hyf.util.showComponent(component, animate);
            }
        }
    }
}

/**
 * Generic utility function for hiding a specific component.
 * @param component {Group_Name | Field_Name} the field, group or HTML component (or its ID) to show/hide
 * @param animate {boolean} (optional) if true the hide will be performed gradually, otherwise it will disappear instantly.
 * @author Hyfinity Limited
 */
hyf.util.hideComponent = function(component, animate)
{
    if (typeof(component) == 'string')
        component = document.getElementById(component);
    if (component == null)
        return;

    if ((animate == true) && (typeof(dojo) != 'undefined') && (typeof(dojo.fx) != 'undefined'))
    {
        dojo.fx.wipeOut({node: component, duration: 250}).play();
    }
    else
    {
        if ((typeof(component._oldCSSDisplay) == undefined) && (hyf.util.getCurrentStyle(component, 'display') != 'none'))
            component._oldCSSDisplay = hyf.util.getCurrentStyle(component, 'display');

        component.style.visibility = 'hidden';
        component.style.display = 'none';
    }
}

/**
 * Generic utility function for showing a specific component.
 * @param component {Group_Name | Field_Name} the field, group or HTML component (or its ID) to show/hide
 * @param animate {boolean} (optional) if true the show will be performed gradually, otherwise it will appear instantly.
 * @author Hyfinity Limited
 */
hyf.util.showComponent = function(component, animate)
{
    if (typeof(component) == 'string')
        component = document.getElementById(component);
    if (component == null)
        return;

    //If the component is being hidden by a display setting in an external
    //css file then just clearing the inline display setting, or using the dojo
    //wipeIn command does not work correctly
    //Therefore we need to remove the class that is hiding it, and switch it to using
    //inline styles to hide.
    //This assumes use of one of the specific hide classes in the provided CSS file,
    //and may not work correclty if a different class name is being used.
    if ((hyf.util.getCurrentStyle(component, 'display') == 'none') && (typeof(dojo) != 'undefined'))
    {
        dojo.removeClass(component, 'hide');
        dojo.removeClass(component, 'hidden');
        component.style.display = 'none';
    }

    if ((animate == true) && (typeof(dojo) != 'undefined') && (typeof(dojo.fx) != 'undefined'))
    {
        dojo.fx.wipeIn({
                    node: component,
                    duration: 250,
                    onEnd: function() { hyf.util.fixWidgetSizes(component); }
                }).play();
    }
    else
    {
        component.style.visibility = 'visible';
        if (typeof(component._oldCSSDisplay) != 'undefined')
            component.style.display= component._oldCSSDisplay;
        else
            component.style.display = '';

        hyf.util.fixWidgetSizes(component);
    }
}

/**
 * Checks if the given container contains any dojo widgets, and if so calls resize on them.
 * This is needed because the widgets do not always correctly render themselves
 * if they were not visible when first shown.
 * @param container The HTML component (e.g. div) to check for widgets.
 * @private
 * @author Hyfinity Limited
 */
hyf.util.fixWidgetSizes = function(container)
{
    var widgets = hyf.util.getDijitWidgets(container);
    for (var i = 0; i < widgets.length; ++i)
    {
        var widget = widgets[i];
        if (typeof(widget.resize) == 'function')
            widget.resize();
    }

    //also check for any auto expanding textareas to make sure they appear correct
    dojo.query('.autoExpandTextarea', container).forEach(function(item){
            item.onkeyup();
    });
}

/**
 * Utility function for disabling a component
 *
 * @param component {Field_Name} the field or HTML component (or its ID) to disable
 * @private
 * @author Hyfinity Limited
 */
hyf.util.disableComponent = function(component)
{
    if (component == null)
        return;

    component.disabled=true;
}

/**
 * Utility function for enabling (stop disabling) a component
 *
 * @param component {Field_Name} the field or HTML component (or its ID) to disable
 * @private
 * @author Hyfinity Limited
 */
hyf.util.enableComponent = function(component)
{
    if (component == null)
        return;

    component.disabled=false;
}

/**
 * Returns the current value for the given style property on the given object
 * @private
 * @author Hyfinity Limited
 */
hyf.util.getCurrentStyle = function(object, property)
{
    var currentDisplay;
    if (window.getComputedStyle)
        currentDisplay = document.defaultView.getComputedStyle(object,null).getPropertyValue(property);
    else if (object.currentStyle)
        currentDisplay = object.currentStyle[property];
    return currentDisplay;
}

/** Stores the space in px to allow for the browser scrollbar.
 * @private
 * @author Hyfinity Limited
 */
hyf.util.scrollbarWidth = 20;

/**
 * Restricts the vertical size of the specified table so that only the given number of rows are displayed
 * at one time, and adds scrollbars to see all the data.
 * @param row The DOM object for a row in the table
 * @param container The DOM object for the DIV containing the table (and nothing else)
 * @param body The DOM object representing the TBODY elemnt of the table.
 * @param headings The DOM object representing the heading row (TR) of the table.
 * @param numRows An integer value indicating the numebr of rows to display.
 * @private
 * @author Hyfinity Limited
 */
hyf.util.setTableScrolling = function(row, container, body, headings, numRows)
{
    if ((row == null) || (container == null) || (body == null) || (headings == null))
    {
        //required components of the table are not present, so return
        return;
    }
    var rowHeight = hyf.util.getComputedHeight(row);
    var headingsHeight = hyf.util.getComputedHeight(headings);

    if (dojo.isIE < 8) //ie versions upto 7 support CSS expressions which makes this quite easy
    {
        container.style.overflow = 'hidden';
        container.style.overflowY = 'auto';
        var newHeight = ((rowHeight * numRows) + headingsHeight);
        if (newHeight < hyf.util.getComputedHeight(container))
        {
            var table = body.parentNode;
            table.style.marginTop = 0;
            container.style.height = newHeight + 'px';
            headings.style.setExpression('top', "document.getElementById('" + container.getAttribute('id') +"').scrollTop - 2");
            headings.style.position = 'relative';
            headings.style.left = 0;

            //make sure the scrollbar appaears next to the table (if not full width)
            if ((hyf.util.getComputedWidth(headings) + hyf.util.scrollbarWidth) < hyf.util.getViewportSize().width)
                container.style.width = hyf.util.getComputedWidth(headings) + hyf.util.scrollbarWidth;
            else
                container.style.width = hyf.util.getComputedWidth(headings);
        }
    }
    else if (dojo.isFF < 4) //firefox before version 4 supported scrollable tbody which makes this easy
    {
        var newHeight = (rowHeight * numRows);
        if (newHeight < hyf.util.getComputedHeight(body))
        {
            body.style.overflow = 'auto';
            body.style.overflowX = 'hidden';
            body.style.height = newHeight + 'px';
        }
    }
    else/* if (dojo.isWebKit || (dojo.isFF >= 4) || (dojo.isIE >= 8))*/
    {
        var newHeight = (rowHeight * numRows);
        if (newHeight < hyf.util.getComputedHeight(body))
        {
            //try and insert a new scrollable container, and position the heding row outside this
            var newContainer = document.createElement('div');
            //move existing container children into new container
            while (container.hasChildNodes())
            {
                newContainer.appendChild(container.firstChild);
            }

            container.appendChild(newContainer);

            container.style.position = 'relative';
            container.style.paddingTop = headingsHeight + 'px';
            newContainer.style.overflow = 'scroll';
            newContainer.style.overflowX = 'hidden';
            newContainer.style.height = newHeight + 'px';

            //set a specific width on each heading, as once it is moved outside, each one
            //will lose the connection to the table data
            if (dojo.isIE == 8)
            {
                for (var i = 0; i < headings.cells.length; i++)
                {
                    var headerCell = headings.cells.item(i);
                    var width = hyf.util.getComputedWidth(headerCell);

                    var dataCell = body.rows.item(0).cells.item(i);

                    headerCell.style.width = width;
                    dataCell.style.width = width;
                    headerCell.style.paddingLeft = dataCell.style.paddingLeft;
                    headerCell.style.paddingRight = dataCell.style.paddingRight;
                }
            }
            else
            {
                dojo.query('th', headings).forEach("item.style.width = hyf.util.getComputedWidth(item) + 'px';");
                dojo.query('td', row).forEach("item.style.width = hyf.util.getComputedWidth(item) + 'px';");
            }

            headings.style.borderSpacing = '0px';
            headings.style.position = 'absolute';
            headings.style.top = '0px';

            var headingWidth = hyf.util.getComputedWidth(headings);
            var table = body.parentNode;
            table.style.width = headingWidth;

            //make sure the scrollbar appaears next to the table (if not full width)
            if ((headingWidth + hyf.util.scrollbarWidth) < hyf.util.getViewportSize().width)
                container.style.width = (headingWidth + hyf.util.scrollbarWidth) + 'px';
        }
    }
    /*else
    {
        //fallback is to just scroll the container, and not keep the table headings fixed
        container.style.overflow = 'scroll';
        container.style.overflowX = 'auto';
        body.style.overflowX = 'hidden';
        var newHeight = (rowHeight * numRows) + headingsHeight;
        if (newHeight < hyf.util.getComputedHeight(container))
        {
            container.style.height = newHeight + 'px';
            container.style.width = (hyf.util.getComputedWidth(headings) + 20) + 'px';
        }
    }*/
}

/**
 * Returns the current height of the given source component
 * @param source The DOM component to return the current computed height for
 * @return {float} containing the current height value (pixels)
 * @private
 * @author Hyfinity Limited
 */
hyf.util.getComputedHeight = function(source)
{
    var compHeight
    if (dojo.isIE)
    {
        compHeight = source.offsetHeight;
    }
    else
    {
        compHeight = hyf.util.getCurrentStyle(source,"height");
    }
    return parseFloat(compHeight);
}

/**
 * Returns the current width of the given source component
 * @param source The DOM component to return the current computed width for
 * @return a float containing the current width value (pixels)
 * @private
 * @author Hyfinity Limited
 */
hyf.util.getComputedWidth = function(source)
{
    var compWidth
    if (dojo.isIE < 9)
    {
        compWidth = source.offsetWidth;
    }
    else
    {
        compWidth = hyf.util.getCurrentStyle(source,"width");
    }
    //special handling for opera which seems to return 0 from getCurrentStyle for the width for some reason.
    if ((parseFloat(compWidth) == 0) && (source.offsetWidth) && (source.offsetWidth > 0))
        compWidth = source.offsetWidth;

    return parseFloat(compWidth);
}


/**
 * Returns an object containing the coordinates of the mouse from the given event.
 * @param evt {object} The event object to get the mouse coordinates for. Can use the objEventSource.event within the script fragment.
 * @return {object} The returned coords object will have two properties, 'x' and 'y' in pixels.
 *
 * @author Hyfinity Limited
 */
hyf.util.getMouseCoords = function(e, base)
{
    var coords= new Object();

    var evt
    if (!e)
    {
        evt = window.event;
    }
    else //netscape
    {
        evt = e;
    }

    //get the mouse coords
    if (evt.pageX || evt.pageY)
    {
        //Gecko based
        coords.x = evt.pageX;
        coords.y = evt.pageY;
    }
    else if (evt.clientX || evt.clientY)
    {
        coords.x = evt.clientX
        coords.y = evt.clientY
        if ((document.body) && (document.body.scrollLeft || document.body.scrollTop))
        {
            //IE 4, 5, and 6 (Non standards compliant mode)
            coords.x += document.body.scrollLeft;
            coords.y += document.body.scrollTop;
        }
        else if ((document.documentElement) &&
                 ((document.documentElement.scrollLeft) ||
                 (document.documentElement.scrollTop)))
        {
            //IE 6 (Standards compliant mode)
            coords.x += document.documentElement.scrollLeft;
            coords.y += document.documentElement.scrollTop;
        }
    }

    if ((typeof(base) != 'undefined') && (typeof(base.x) != 'undefined') && (typeof(base.y) != 'undefined'))
    {
        coords.x += base.x;
        coords.y += base.y;
    }

    return coords;

}

/**
 * Returns the location of the left of the given object.
 * If the screen parameter is set to true, then the returned value will be in screen coords.
 * ie it adjusts for scrollable regions, to allow comparison with mouse coordinates
 * returned from getMouseCoords method
 * @private
 * @author Hyfinity Limited
 */
hyf.util.getLeftPosition = function(obj, screen)
{
    var ol=obj.offsetLeft;
    while ((obj=obj.offsetParent) != null)
    {
        ol += obj.offsetLeft;
        if (screen)
        {
            if (obj.scrollLeft != 0)
            {
                ot -= obj.scrollLeft;
            }
        }
    }
    return ol;
}

/**
 * Returns the location of the right of the given object.
 * If the screen parameter is set to true, then the returned value will be in screen coords.
 * ie it adjusts for scrollable regions, to allow comparison with mouse coordinates
 * returned from getMouseCoords method
 * @private
 * @author Hyfinity Limited
 */
hyf.util.getRightPosition = function(obj, screen)
{
    return hyf.util.getLeftPosition(obj, screen) + hyf.util.getComputedWidth(obj);
}

/**
 * Returns the location of the top of the given object.
 * If the screen parameter is set to true, then the returned value will be in screen coords.
 * ie it adjusts for scrollable regions, to allow comparison with mouse coordinates
 * @return returned from getMouseCoords method
 * @private
 * @author Hyfinity Limited
 */
hyf.util.getTopPosition = function(obj, screen)
{
    var ot=obj.offsetTop;
    while ((obj=obj.offsetParent) != null)
    {
        ot += obj.offsetTop;
        if (screen)
        {
            if (obj.scrollTop != 0)
            {
                ot -= obj.scrollTop;
            }
        }
    }
    return ot;
}

/**
 * Returns the current location coordinates of the given object.
 * The returned object will have four properties, 'x' and 'y' coordinates which give
 * the position of the top left corner, and 'width' and 'height'
 * @param component {hyf.FieldName | hyf.GroupName} the field or group on the page.
 * @param screen {boolean} (optional) If the screen parameter is set to true, the returned values are in 'screen coords', ie they are
 * adjusted for scrollable regions, to allow comparison with the mouse coordinates returned from
 * the getMouseCoords method.
 *
 * @author Hyfinity Limited
 */
hyf.util.getComponentPosition = function(obj, screen)
{
    if (typeof(obj) == 'string')
    {
        obj = document.getElementById(obj);
    }
    var objCoords = new Object();
    objCoords.x = hyf.util.getLeftPosition(obj, screen);
    objCoords.y = hyf.util.getTopPosition(obj, screen);
    objCoords.width = hyf.util.getComputedWidth(obj);
    objCoords.height = hyf.util.getComputedHeight(obj);
    return objCoords;
}

/**
 * Returns the size of the available window viewport
 * @return an object containing two properties, width and height.
 * @private
 * @author Hyfinity Limited
 */
hyf.util.getViewportSize = function()
{
    var size = new Object();
    if (window.innerWidth)
    {
        size.width = window.innerWidth;
        size.height = window.innerHeight;
    }
    else if (document.documentElement && document.documentElement.clientWidth)
    {
        size.width = document.documentElement.clientWidth;
        size.height = document.documentElement.clientHeight;
    }
    else if (document.body && document.body.clientWidth)
    {
        size.width = document.body.clientWidth;
        size.height = document.body.clientHeight;
    }
    return size;
}

/**
 * Returns the closest previous sibling to the given element
 * that is an element node. ie text nodes are ignored
 * @private
 * @author Hyfinity Limited
 */
hyf.util.getPreviousElementSibling = function(elem)
{
    var prev = elem.previousSibling;
    //make sure we return an element type node, not a text node
    while (prev && prev.nodeType != 1)
    {
        prev = prev.previousSibling;
    }

    return prev;
}

/**
 * Returns the closest folowing sibling to the given element
 * that is an element node. ie text nodes are ignored
 * @private
 * @author Hyfinity Limited
 */
hyf.util.getNextElementSibling = function(elem)
{
    var prev = elem.nextSibling;
    //make sure we return an element type node, not a text node
    while (prev && prev.nodeType != 1)
    {
        prev = prev.nextSibling;
    }

    return prev;
}


/**
 * Returns a boolean value indicating whether or not the given field is currently hidden
 * @param field {Field_Name} The HTML field to check.
 * @return true if the given field is within a hidden (display:none) component, false otherwise
 *
 * @author Hyfinity Limited
 */
hyf.util.checkFieldHidden = function(field)
{
    if (typeof(field) == 'string')
    {
        field = document.getElementById(field);
    }
    if(hyf.util.findFieldHiddenParents(field).length > 0)
        return true;
    else
        return false;

}

/**
 * Searches the parents of the given field, and looks for any hidden (display:none) components.
 * This then returns an array of all the hidden components found.
 * If this field is not hidden, then an empty array will be returned
 * @param field The HTML field to check the parents of.
 * @param matches This param should not be passed in.
 * @private
 * @author Hyfinity Limited
 */
hyf.util.findFieldHiddenParents = function(field, matches)
{
    if (typeof(matches) == 'undefined')
        matches = new Array();

    if (field.nodeType == 9)  //have reached the document node
    {
        return matches
    }

    var disp = hyf.util.getCurrentStyle(field,"display");

    if (disp == 'none')
    {
        matches[matches.length] = field;
    }

    if ((typeof(field.parentNode) != 'undefined') && (field.parentNode != null))
    {
        return hyf.util.findFieldHiddenParents(field.parentNode, matches);
    }
    else
    {
        return matches;
    }
}

/**
 * Searches through the parents of the given field to determine whether it is contained
 * in any layout containers that allow the user to choose whether the field is visble or not.
 * This currently checks for tabs, accordions, and collapsible content controls.
 * @param field {Field_Name} The HTML field to check the parents of.
 * @param matches {object} This param should not be passed in.
 * @return An array of all the parent elements found the represent these controls.
 *       If the field is not within any of these containers then an empty array is returned.
 * @private
 * @author Hyfinity Limited
 */
hyf.util.findFieldLayoutContainerParents = function(field, matches)
{
    if (typeof(matches) == 'undefined')
        matches = new Array();

    if (field.nodeType == 9)  //have reached the document node
    {
        return matches
    }

    //first check for tabs
    if (field.getAttribute('_use') == 'tabPane')
    {
        //check that there is actually a tab button for this tab pane
        var tabFieldId = field.getAttribute('_tabField');
        if (document.getElementById(tabFieldId) != null)
            matches[matches.length] = field;
    }

    //check for collapsible content
    if (dojo.hasClass(field, 'collapsibleContent') || dojo.hasClass(field, 'collapsedContent'))
    {
        //Check for toggle button
        if (dojo.query("a.toggleHidden, a.toggleVisible", field.parentNode).length > 0)
            matches[matches.length] = field;
    }

    //check for accordion panes
    var dojoType = field.getAttribute('dojoType');
    if ((dojoType != null) && ((dojoType == 'dijit.layout.ContentPane') || (dojoType == 'dijit.layout.AccordionPane')))
    {
        var widget = dijit.byNode(field);
        if (widget != null)
        {
            //make sure widget is part of an accordion
            if (field.parentNode && field.parentNode.parentNode &&
                dijit.getEnclosingWidget(field.parentNode.parentNode).declaredClass == 'dijit.layout.AccordionContainer')
                    matches[matches.length] = field;
        }
    }


    if ((typeof(field.parentNode) != 'undefined') && (field.parentNode != null))
    {
        return hyf.util.findFieldLayoutContainerParents(field.parentNode, matches);
    }
    else
    {
        return matches;
    }
}

/**
 * Adjusts the visibility of all the provided containers so that they are all set to the visible state.
 * This should be passed the output from the findFieldLayoutContainerParents method above.
 * This currently supports adjusting tabs, accordions, and collapsible content containers.
 * @param containers An array of all the container elements to make visible.
 * @private
 * @author Hyfinity Limited
 */
hyf.util.makeLayoutContainersVisible = function(containers)
{
    //loop through each container element
    for (var i = containers.length - 1; i > -1; --i)
    {
        var c = containers[i];

        //check if it is a tab container
        if (c.getAttribute('_use') == 'tabPane')
        {
            var tabFieldId = c.getAttribute('_tabField');
            if (tabFieldId != null)
            {
                var tabBtn = document.getElementById(tabFieldId);
                //click the tab button to make sure the correct tab is visible.
                if (tabBtn != null)
                {
                    tabBtn.onclick();
                }
            }
        }

        //check if it is a collapsible content
        if (dojo.hasClass(c, 'collapsibleContent') || dojo.hasClass(c, 'collapsedContent'))
        {
            //make sure this group is hidden
            if (hyf.util.getCurrentStyle(c, "display") == 'none')
            {
                //find the toggel button
                var toggleBtn = dojo.query("a.toggleHidden, a.toggleVisible", c.parentNode)[0];
                if (toggleBtn != null)
                {
                    hyf.collapsiblecontent.toggle(toggleBtn);
                }
            }
        }

        //check if it is an accordion
        var dojoType = c.getAttribute('dojoType');
        if ((dojoType != null) && ((dojoType == 'dijit.layout.ContentPane') || (dojoType == 'dijit.layout.AccordionPane')))
        {
            var pane = dijit.byNode(c);
            if (pane != null)
            {
                //make sure widget is part of an accordion
                if (c.parentNode && c.parentNode.parentNode)
                {
                    var acc = dijit.getEnclosingWidget(c.parentNode.parentNode);
                    if (acc != null)
                    {
                        acc.selectChild(pane);
                    }
                }
            }
        }
    }
}


/**
 * Checks to see if the provided parent component does actually contain the
 * child element within it a some level.
 * If both params are the same, then this will return true.
 * @param child {Field_Name | Group_Name} The child element to check.
 * @param parent {Group_Name} The HTML component to check if it contains child.
 * @return {boolean} indicating whether or not parent contains child.
 * @private
 * @author Hyfinity Limited
 */
hyf.util.isParent = function(child, parent)
{
    if (child.nodeType == 9)  //have reached the document node
    {
        return false
    }
    if (child == parent)
    {
        return true
    }
    else if ((typeof(child.parentNode) != 'undefined') && (child.parentNode != null))
    {
        return hyf.util.isParent(child.parentNode, parent);
    }
    else
    {
        return false;
    }
}

/**
 * Attempts to get the current value for the given field.
 * This will attempt to determine the type of the given field name,
 * by checking dojo widigets, the form elements collection, and
 * display only containers with the given id.
 * @param fieldName {Field_Name} The name of the field to get the value for.
 * @param getDisplayValue {boolean} (optional) If provided and set to true, the returned value will be the displayed value
 *                  for the control rather than the underlying data value where appropriate (e.g. select controls)
 * @return {string | array} containing the current value, or null if the field couldn't be found.
 *         For multiple selection controls, an array of all current values will be returned.
 *
 * @author Hyfinity Limited
 */
hyf.util.getFieldValue = function(fieldName, getDisplayValue)
{
    if (typeof(getDisplayValue) != 'boolean')
        getDisplayValue = false;

    //first check for a dojo widget
    if ((typeof(dijit) != 'undefined') && (typeof(dijit.byId) == 'function') && (dijit.byId(fieldName) != null))
    {
        if (getDisplayValue)
            return dijit.byId(fieldName).get('displayedValue');
        else
            return dijit.byId(fieldName).get('value');
    }

    //check for form controls
    var nameMatches = dojo.query('*[name=' + fieldName + ']');
    if (nameMatches.length > 0)
    {
        var controlFound = true;
        var values = new Array();
        if (nameMatches.length == 1)
        {
            var entry = nameMatches[0];

            if ((typeof(entry.type) == 'undefined') || (entry.type == ''))
            {
                //not actually a field control
                controlFound = false;
            }
            else
            {
                if (entry.type == 'select-multiple')
                { //for multi select we need to find all selected values
                    for (var i = 0; i < entry.options.length; ++i)
                    {
                        if (entry.options[i].selected)
                        {
                            if (getDisplayValue)
                                values[values.length] = entry.options[i].text;
                            else
                                values[values.length] = entry.options[i].value;
                        }
                    }
                }
                else if (entry.type == 'select-one')
                {
                    if (getDisplayValue)
                        values[values.length] = entry.options[entry.selectedIndex].text;
                    else
                        values[values.length] = entry.value;
                }
                else if (entry.type == 'checkbox')
                { //for single checkbox, if not ticked, then we should return the unticked value
                    if (entry.checked)
                        values[values.length] = entry.value;
                    else
                        values[values.length] = document.getElementById(entry.id + '_value_if_not_submitted').value;
                }
                else if ((entry.type == 'textarea') && (entry.className == 'richTextEditor'))
                {
                    //for rich text editor, we need to get the content that is currently in the RichText Control
                    if (tinyMCE)
                        tinyMCE.triggerSave(entry.id);
                    values[values.length] = document.getElementById(entry.id).value;
                }
                else
                {
                    //check for masking
                    if (!getDisplayValue && hyf.validation.isValueMasked(entry))
                        values[values.length] = hyf.validation.getUnmaskedValue(entry);
                    else
                        values[values.length] = entry.value;
                }
            }
        }
        else  //is an array of controls, eg radio or multi check.
        {
            for (var i = 0; i < nameMatches.length; ++i)
            {
                if (nameMatches[i].checked)
                {
                    if (getDisplayValue)
                    {
                        //try and look for a label element for this control to get the
                        //text content of
                        var container = document.getElementById(nameMatches[i].name + '_container');
                        var labels = dojo.query('label[for="'+nameMatches[i].id+'"]', container);
                        if (labels.length > 0)
                        {
                            var label = labels[0];
                            if (typeof(label.innerText) != 'undefined')
                                values[values.length] = label.innerText;
                            else if (typeof(label.textContent) != 'undefined')
                                values[values.length] = label.textContent;
                        }
                        else
                            values[values.length] = nameMatches[i].value;
                    }
                    else
                        values[values.length] = nameMatches[i].value;
                }
            }
        }
        if (controlFound)
        {
            if (values.length == 0)
                return '';
            else if (values.length == 1)
                return values[0];
            else
                return values;
        }
    }

    //finally check for an id
    if (document.getElementById(fieldName) != null)
    {
        //return the text contents of the container.
        var cont = document.getElementById(fieldName);
        if (!getDisplayValue && hyf.validation.isValueMasked(cont))
            return hyf.validation.getUnmaskedValue(cont);
        else if (typeof(cont.innerText) != 'undefined')
            return cont.innerText;
        else if (typeof(cont.textContent) != 'undefined')
            return cont.textContent;
    }

    //couldnt find a match so return null
    return null;
}

/**
 * Attempts to set the current value for the given field.
 * This will attempt to determine the type of the given field name,
 * by checking dojo widigets, the form elements collection, and
 * display only containers with the given id.
 * If the field supports multiple selections, then the provided value will be
 * added in addition to any others
 * @param fieldName {Field_Name} The name of the field to set the value for.
 * @param fieldValue {Field_Value | string} The value to set the field with.
 *
 * @author Hyfinity Limited
 */
hyf.util.setFieldValue = function(fieldName, fieldValue)
{
    //first check for a dojo widget
    if ((typeof(dijit) != 'undefined') && (typeof(dijit.byId) == 'function') && (dijit.byId(fieldName) != null))
    {
        return dijit.byId(fieldName).set('value', fieldValue);
    }

    //check for form controls
    var nameMatches = dojo.query('*[name=' + fieldName + ']');
    if (nameMatches.length > 0)
    {
        var controlFound = true;
        var valueToSet = hyf.util.processDateConstantsForField(fieldValue, nameMatches[0]);

        if (nameMatches.length == 1)
        {
            var entry = nameMatches[0];
            if ((typeof(entry.type) == 'undefined') || (entry.type == ''))
            {
                //not actually a field control
                controlFound = false;
            }
            else
            {
                if (entry.type == 'select-multiple')
                { //for multi select we need to find the matching option to select
                    for (var i = 0; i < entry.options.length; ++i)
                    {
                        if (entry.options[i].value == valueToSet)
                            entry.options[i].selected = true;
                    }
                }
                else if (entry.type == 'checkbox')
                { //for single checkbox, we need to adjust the checked proeprty
                    if (entry.value == valueToSet)
                        entry.checked = true;
                    else
                        entry.checked = false;
                }
                else if ((entry.type == 'textarea') && (entry.className == 'richTextEditor'))
                { //for rich text editor, we need to adjust the content of the RichText Control as well
                    entry.value = valueToSet;
                    if (tinyMCE)
                        tinyMCE.get(fieldName).setContent(valueToSet);
                }
                else
                {
                    entry.value = valueToSet;
                    //check for any masking requirements
                    hyf.validation.applyMask(entry);
                }

                hyf.fireEvent(entry, 'change');
            }
        }
        else  //is an array of controls, eg radio or multi check.
        {
            for (var i = 0; i < nameMatches.length; ++i)
            {
                if (nameMatches[i].value == valueToSet)
                    nameMatches[i].checked = true;

                //TODO: Should we call some kind of onchange function here???
                //Generally we use onclick to detect changes to radio/checkbox controls

            }
        }
        if (controlFound)
            return;
    }

    //finally check for an id
    if (document.getElementById(fieldName) != null)
    {
        //set the text contents of the container.
        var cont = document.getElementById(fieldName);

        var valueToSet = hyf.util.processDateConstantsForField(fieldValue, cont);

        //if the container has existing text node children then we just update the first one
        var textUpdated = false;
        var child = cont.firstChild;
        while (child != null)
        {
            if (child.nodeType == 3) //text node
            {
                if (!textUpdated)
                {
                    child.nodeValue = valueToSet;
                    textUpdated = true;
                    child = child.nextSibling;
                }
                else
                {
                    var oldchild = child;
                    child = child.nextSibling;
                    cont.removeChild(oldchild);
                }
            }
            else
            {
                child = child.nextSibling;
            }
        }

        //otherwise we set the contents of the element, removing any existing content
        if (!textUpdated)
        {
            if (typeof(cont.innerText) != 'undefined')
                cont.innerText = valueToSet;
            else if (typeof(cont.textContent) != 'undefined')
                cont.textContent = valueToSet;
        }

        //check for any masking requirements
        hyf.validation.applyMask(cont);
    }
}

/**
 * Gets the current value of the given display variable.
 * @param dvName {Display_Variable} The name of the display variable to get the value for.
 * @return {string} The requested value, or null if the display varaiable doesn't exist.
 *
 * @author Hyfinity Limited
 */
hyf.util.getDisplayVariableValue = function(dvName)
{
    //all display variables are stored as hidden fields on the page using a defined
    //naming format, so we need to find this field, and get its value here.
    var dvField = document.getElementById('hyf_display_variable_' + dvName);
    if (dvField != null)
    {
        return dvField.value;
    }
    return null;
}

/**
 * Sets the current value of the given display variable.
 * @param dvName {Display_Variable} The name of the display variable to set the value for.
 * @param newValue {string} The new value to give the display variable
 * @return {boolean} true of the value was updated, false if the variable name couldn't be found.
 *
 * @author Hyfinity Limited
 */
hyf.util.setDisplayVariableValue = function(dvName, newValue)
{
    //all display variables are stored as hidden fields on the page using a defined
    //naming format, so we need to find this field, and get its value here.
    var dvField = document.getElementById('hyf_display_variable_' + dvName);
    if (dvField != null)
    {
        dvField.value = newValue;

        //display variable fields are disabled so they are not submitted,
        //but this means that events dont fire on them
        //So need to enable the field temporarily to fire the change.
        dvField.disabled = false;

        //for some reason we need to allow some time after enabling
        //for the fire event to work properly (at least in FF) so we
        //use a timeout here.
        window.setTimeout(function(){
                hyf.fireEvent(dvField, 'change');
                dvField.disabled = true;
        }, 100);

        return true;
    }
    return false;
}

/**
 * Checks if the provided value is actually one of the defined date constants
 * (in hyf.calendar.supportedKeywords).  If so, and the provided field is a date value,
 * then this returns the appropriate date string in the correct format for the field.
 * If not, then the provided value is returned as is.
 * @private
 */
hyf.util.processDateConstantsForField = function(value, field)
{
    if ((field.getAttribute('_type') == 'date') && (typeof(value) == 'string'))
    {
        if ((value.charAt(0) == '$') && (hyf.calendar.supportedKeywords[value.substr(1)] != null))
        {
            var dateToAdjust = new Date();

            if (hyf.calendar.supportedKeywords[value.substr(1)] != '')
                eval(hyf.calendar.supportedKeywords[value.substr(1)]);

            var dateFormat = (field.getAttribute('_display_date_format')) ? field.getAttribute('_display_date_format') : field.getAttribute('_data_date_format');

            value = formatDate(dateToAdjust, dateFormat);
        }
    }

    return value;
}


/**
 * Function to insert HTML content into the specified target container.
 * This also provides the option to evaluate any scripts present in the content
 *
 * @param target The HTML container into which the content will be inserted.
 *               Any existing content in this container will be lost.
 * @param content A string containing the HTML fragment to insert.
 * @param evalScripts an optional boolean flag to indicate whether scripts in the content should be evaluated
 *                  These will be evalauted only if true.
 * @private
 */
hyf.util.insertContent = function(target, content, evalScripts)
{
    //if we dont have a target to insert into then stop
    if ((target == null) || (typeof(target) == 'undefined'))
        return;

    //if we dont have any content to insert then stop
    if ((content == null) || (typeof(content) != 'string'))
        return;

    scriptRegExp = '(?:<script.*?>)((\n|\r|.)*?)(?:<\/script>)';

    //seperate out any script tags from the content passed in.
    var match = new RegExp(scriptRegExp, 'img');
    var pureHTML = content.replace(match, '');
    var scripts  = content.match(match);

    //attempt to destory any existing widgets in the container
    hyf.util.destroyDojoWidgets(target);

    //insert the HTML into the targte location
    target.innerHTML = pureHTML;

    //check if the script fragments that we have stripped out need to be evaluated
    if (evalScripts && scripts != null)
    {
        match = new RegExp(scriptRegExp, 'im');
        setTimeout(function() {
            var onLoadEventsIdentifier = new Date().getTime();
            var onloadEventsRegExp = /hyf.attachEventHandler\s*\(\s*window\s*,\s*['"]onload/img;

            //long variable name to try and avoid conflicts with the scripts being executed.
            for (var script_counter_unique_name = 0; script_counter_unique_name < scripts.length; script_counter_unique_name++)
            {
                try
                {
                    //use the regex again (without the global flag) to remove the actual surrounding script tags
                    var scriptFragment = scripts[script_counter_unique_name].match(match)[1];
                    //check to see if the fragment contains a function definition which needs to
                    //be properly registered with this window
                    var functionStart = scriptFragment.indexOf('function ');
                    while(functionStart != -1)
                    {
                        var functionEnd = scriptFragment.indexOf('(', functionStart + 9);
                        var functionName = scriptFragment.substring(functionStart + 9, functionEnd);
                        if (functionName != '')
                            scriptFragment += '; window.' + functionName + ' = ' + functionName + ';';
                        functionStart = scriptFragment.indexOf('function ', functionEnd);
                    }

                    //check if the script fragment contains any calls to find the last table output,
                    //and if so try and replace this with the actual ID of the table so that this call
                    //will still work correctly.
                    var gltIndex = scriptFragment.indexOf('getLastTableOutput');
                    if (gltIndex != -1)
                    {
                        //find the location of this script fragment in the original HTML
                        var loc = content.indexOf(scripts[script_counter_unique_name]);
                        //find the id of the closest table before this point
                        var tableStart = content.lastIndexOf('<table', loc);
                        if (tableStart != -1)
                        {
                            var idStart = content.indexOf('id=', tableStart);
                            if (idStart != -1)
                            {
                                var idQuote = content.charAt(idStart + 3);
                                var idEnd = content.indexOf(idQuote, idStart + 4);
                                var idString = content.substring(idStart + 4, idEnd);
                                if (idString != '')
                                {
                                    //work out which part of the script fragment we need to replace
                                    var replaceStart = scriptFragment.lastIndexOf('hyf', gltIndex);
                                    var replaceEnd = scriptFragment.indexOf(')', gltIndex);
                                    //replace the section of the script fragment
                                    scriptFragment = scriptFragment.substring(0, replaceStart) +
                                                     "document.getElementById('" + idString + "')" +
                                                     scriptFragment.substring(replaceEnd + 1);
                                }
                            }
                        }
                    }

                    //check for any calls to attachEventHandler for onload events
                    //if so we need to add our unique identifier for this insertContent call
                    //so that we can evaluate these onload events at the right point below
                    while (onloadEventsRegExp.test(scriptFragment))
                    {
                        scriptFragment = scriptFragment.substr(0, onloadEventsRegExp.lastIndex)
                                                + '-' + onLoadEventsIdentifier
                                                + scriptFragment.substr(onloadEventsRegExp.lastIndex);
                    }

                    eval(scriptFragment);
                }
                catch(e)
                {
                    //ignore
                }
            }

            //call the content inserted hooks
            hyf.hooks.contentInserted(target);

            //check for any dojo widgets in the inserted HTML
            if ((typeof(dojo) != 'undefined') && (typeof(dojo.parser) != 'undefined'))
                dojo.parser.parse(target);

            //call any 'onload' events found in the script
            if (hyf.afterLoadOnLoadFunctions[onLoadEventsIdentifier])
            {
                for(var i = 0; i < hyf.afterLoadOnLoadFunctions[onLoadEventsIdentifier].length; ++i)
                {
                    hyf.afterLoadOnLoadFunctions[onLoadEventsIdentifier][i]();
                }

                delete hyf.afterLoadOnLoadFunctions[onLoadEventsIdentifier];
            }

        }, 10);
    }
}


/**
 * Creates an object containing the contents of all the input fields within the specified container
 *
 * @param cont The container DOM object to process.
 * @param actionName (optional) The name of the action being called.  If present, then the associated instance
 *                              field will be included in the returned object.
 * @return The created associative array mapping param names to values for submitting in a dojo AJAX request.
 *
 * @private
 */
hyf.util.encodeContainer = function(cont, actionName)
{
    if (cont == null || typeof(cont) == 'undefined')
        return;

    var data = new Object();

    var inputs = cont.getElementsByTagName('input');

    for (var i = 0; i < inputs.length; ++i)
    {
        //look at the type of input
        switch (inputs[i].type)
        {
            case 'file'     :   //cant handle this via ajax so just ignore
                                break;
            case 'radio'    :
            case 'checkbox' :   if (inputs[i].checked)
                                {
                                    //handle multi check fields
                                    if (data[inputs[i].name])
                                    {
                                        if (typeof(data[inputs[i].name]) == 'string')
                                        {
                                            var values = new Array();
                                            values.push(data[inputs[i].name]);
                                            data[inputs[i].name] = values;
                                        }
                                        data[inputs[i].name].push(inputs[i].value);

                                    }
                                    else
                                    data[inputs[i].name] = inputs[i].value;
                                }
                                break;
            //QUESTION: How should we handle these types?  Should they be submitted?
            case 'image'    :   break;
            case 'button'   :   break;
            case 'reset'    :   break;
            case 'submit'   :   break;
            default         :   if (hyf.validation.isValueMasked(inputs[i]))
                                    data[inputs[i].name] = hyf.validation.getUnmaskedValue(inputs[i]);
                                else
                                    data[inputs[i].name] = inputs[i].value;
        }
    }

    inputs = cont.getElementsByTagName('textarea');
    for (var i = 0; i < inputs.length; ++i)
    {
        if (hyf.validation.isValueMasked(inputs[i]))
            data[inputs[i].name] = hyf.validation.getUnmaskedValue(inputs[i]);
        else
            data[inputs[i].name] = inputs[i].value;
    }

    inputs = cont.getElementsByTagName('select');
    for (var i = 0; i < inputs.length; ++i)
    {
        if (inputs[i].type == 'select-multiple')
        {
            var values = new Array();
            for (var j = 0; j < inputs[i].options.length; ++j)
            {
                if (inputs[i].options[j].selected)
                {
                    values.push(inputs[i].options[j].value);
                }
            }
            if (values.length > 0)
                data[inputs[i].name] = values;
        }
        else
        {
            data[inputs[i].name] = inputs[i].value;
        }
    }

    //need to add the language value as this wont be included in the encoded group
    for (var i = 0; i < document.forms.length; ++i)
    {
        if (typeof(document.forms[i].Language) != 'undefined')
        {
            data['Language'] = document.forms[0].Language.value;
            break;
        }
    }

    //add the appropriate xform instance field if the action value has been provided
    if (typeof(actionName) != 'null')
    {
        //First check for the simple case where there is an instance field for this specific action.
        var xif = document.getElementById('xgf_xform_instance_xga_' + actionName);
        if (xif != null)
            data[xif.name] = xif.value;
        else
        {
            //if not then loop through all the fields to see if we have an instance structure to use
            var possInstances = document.getElementsByTagName('input');

            var possInstance = null;

            for (var i = 0; i < possInstances.length; ++i)
            {
                if (possInstances[i].type == 'hidden')
                {
                    if ((possInstances[i].name == 'xform_instance') && (possInstance == null))
                        possInstance = possInstances[i]
                    else if (possInstances[i].name.indexOf('xgf_xform_instance') == 0)
                    {
                        if ((possInstances[i].name + '_').indexOf('_xga_' + actionName + '_') != -1)
                        {
                            possInstance = possInstances[i];
                            break;
                        }
                    }
                }
            }
            if (possInstance != null)
                data[possInstance.name] = possInstance.value;
        }
    }

    return data;
};


/**
 * Resets all the form controls in the given container.
 * Depending on the provided mode value, this will either 'reset' their values to those shown when the
 * page was first displayed, or 'clear' the values completely.
 * @param container {Group_Name} The HTML container whose form controls should be processed.
 * @param mode {string} (optional) Indicates which type of reset should apply.
 * Either 'reset' (default) or 'clear'.
 * 'reset' will put the previous values from when the page was loaded including defaults.
 * 'clear' will remove all the values.
 *
 * @author Hyfinity Limited
 */
hyf.util.resetContainer = function(container, mode)
{
    if (typeof(container) == 'string')
        container = document.getElementById(container);
    if ((container == null) || (typeof(container) == 'undefined'))
        return;
    if ((typeof(mode) == 'undefined') || ((mode != 'reset') && (mode != 'clear')))
       mode = 'reset';

    //handle the special case of the form node being passed in, which provides a reset method.
    if ((typeof(container.reset) == 'function') && (mode == 'reset'))
        container.reset()
    else
    {
        var controls = new Array();
        var inputs = container.getElementsByTagName('input');
        for (var i = 0; i < inputs.length; ++i)
            controls[controls.length] = inputs[i];
        inputs = container.getElementsByTagName('select');
        for (var i = 0; i < inputs.length; ++i)
            controls[controls.length] = inputs[i];
        inputs = container.getElementsByTagName('textarea');
        for (var i = 0; i < inputs.length; ++i)
            controls[controls.length] = inputs[i];

        for (var i = 0; i < controls.length; ++i)
        {
            //look at the type of input
            if (mode == 'clear')
            {
                switch (controls[i].type)
                {
                    case 'image'    :   break;
                    case 'button'   :   break;
                    case 'reset'    :   break;
                    case 'submit'   :   break;
                    case 'file'     :   break;
                    case 'select-one':  //want select one control to default to the first value?
                                        controls[i].selectedIndex = 0;
                                        break;
                    case 'select-multiple': controls[i].selectedIndex = -1;
                                            break;
                    case 'radio'    :
                    case 'checkbox' :   controls[i].checked = false;
                                        break;
                    default         :   controls[i].value = '';
                }
            }
            else
            {
                switch (controls[i].type)
                {
                    case 'image'    :   break;
                    case 'button'   :   break;
                    case 'reset'    :   break;
                    case 'submit'   :   break;
                    case 'file'     :   break;
                    case 'select-one':
                    case 'select-multiple': for (var op = 0; op < controls[i].options.length; ++op)
                                            {
                                                controls[i].options[op].selected = controls[i].options[op].defaultSelected;
                                            }
                                            break;
                    case 'radio'    :
                    case 'checkbox' :   controls[i].checked = controls[i].defaultChecked;
                                        break;
                    default         :   controls[i].value = controls[i].defaultValue;
                }
            }
        }
    }
}

/**
 * Attempts to destory any dojo widgets that are present in the specified container
 * @param container The HTML container to destroy dojo widgets within. If not provided then the
 *              document body will be used instead, to destory any widgets on the page.
 * @private
 * @author Hyfinity Limited
 */
hyf.util.destroyDojoWidgets = function(container)
{
    var widgets = hyf.util.getDijitWidgets(container);
    for (var i = 0; i < widgets.length; ++i)
    {
        var widget = widgets[i];
        widget.destroyRecursive();
        delete widget;
    }
}



/*
 * The hyf.textarea namespace contains all functionality relating to textarea field controls.
 */
hyf.textarea =
{
    desc : "Handles functionality relating to textareas, eg auto expanding fields."
};

/**
 * Handles automatically updating the size of the supplied textarea to support the current content.
 *
 * @param evt The event object that triggered the call.
 * @param field The textarea object to adjust
 * @param min The minim number of rows to show in the textarea.
 * @param max The maximum number of rows to expand the textarea to.
 * @private
 * @author Hyfinity Limited
 */
hyf.textarea.adjustHeight = function(evt, field, min, max)
{
    if ((evt != null) && (typeof(evt) != 'undefined'))
        var charCode = (evt.which) ? evt.which : event.keyCode;

    //ignore the arrow keys and shift/ctrl key
    if ((charCode >= 37 && charCode <= 40) || (charCode == 16) || (charCode == 17) || (evt && evt.ctrlKey))
    {
        return;
    }

    if ((max == null) || typeof(max) == 'undefined')
        max = Number.MAX_VALUE;
    if ((min == null) || typeof(min) == 'undefined')
        min = 1;

    //ensure the textarea has been correctly initiated
    if (!field._clone)
        hyf.textarea.initTextarea(field);
    else if ((field._lineHeightUpdateRequired == true) || (field._lineHeight == 0))
    {
        hyf.textarea.determineLineHeight(field);
        field._lineHeightUpdateRequired = false;
    }

    if (field._lineHeight == 0)
        return;

    //out the current text into the clone field to see how big it is
    field._clone.value = field.value;

    var newHeight = field._clone.scrollHeight;
    var minHeight = field._lineHeight * min;
    var maxHeight = field._lineHeight * max;

    //only make any changes if the new height is different to the last one we found
    if (newHeight != field._lastHeight)
    {
        field._lastHeight = newHeight;
        if (newHeight > maxHeight)
        {
            //stop resizing, and allow the vertical scrollbar to appear
            field.style.overflowY = 'auto';
            field.style.height = maxHeight + 'px';
        }
        else if (newHeight < minHeight)
        {
            //dont resize
            field.style.overflowY = 'hidden';
            field.style.height = minHeight + 'px';
        }
        else
        {
            field.style.overflowY = 'hidden';
            field.style.height = newHeight + 'px';
        }
    }
};



/**
 * Initialises the given textarea so that the expanding functionality
 * will work correctly.
 * @param ta The textarea to initialise.
  * @private
 * @author Hyfinity Limited
 */
hyf.textarea.initTextarea = function(ta)
{
    //remove any current rows setting, as we will instead be adjusting the
    //height of the control
    ta.removeAttribute('rows');

    //make a clone of the textarea that we can use to determine what height
    //the main textarea should be
    var clone = ta.cloneNode(true);
    ta._clone = clone;

    dojo.addClass(ta, 'autoExpandTextarea');

    //ensure the clone doesnt appear or affect the working of the page
    clone.removeAttribute('id');
    clone.removeAttribute('name');
    clone.disabled = true;
    dojo.style(clone, {overflowY : 'auto', position: 'absolute', top: '0px', left : '-99999px', height : 'auto'});
    ta.parentNode.appendChild(clone);

    //try and guestimate the line height of the field, as there is no cross browser way to get this easily
    hyf.textarea.determineLineHeight(ta);
    //in IE the first guess of the line height doesnt always come out correctly,
    //so we try again at a later point
    if (dojo.isIE)
    {
        ta._lineHeightUpdateRequired = true;
        window.setTimeout(function(){ta.onkeyup()}, 5);
    }
    ta._lastHeight = 0;

    //just to be sure, add another call to update the size, on focus of the field.
    dojo.connect(ta, 'onfocus', ta, new Function('this.onkeyup()'));
}

/**
 * Makes a guess at the line height value for the provided
 * textarea by putting temporary content into the clone
 * and seeing how big it gets.
 * This value is then stored against the textarea
 * @private
 * @author Hyfinity Limited
 */
hyf.textarea.determineLineHeight = function(ta)
{
    ta._clone.value = 'a\nb\nc\nd\ne';
    ta._lineHeight = (ta._clone.scrollHeight / 5);
}


/**
 * Checks the provided fieldId to see if it looks like it actually also contains the
 * repeatId info, and if so try and split it out
 * @param fieldId The full Id to split
 * @return An object containing fieldName and repeatId properties.  If the provided Id could not be split
 *         then the fieldName property will match that passed in, and the repeatId will be an empty string.
 * @private
 */
hyf.util.splitFullFieldId = function(fieldId)
{
    var retObj = {fieldName: fieldId, repeatId: ''};

    var r = new RegExp('^(\\w*[a-zA-Z])([0-9]+)(\\w+)$');
    var match = r.exec(fieldId);
    if (match != null)
    {
        retObj.repeatId = match[1] + match[2];
        retObj.fieldName = match[3];
    }

    return retObj;
}


/*
 * Namespace for all the calendar related functionality
 * The configuration options for each calendar should be populated by script fragments in the actual page.
 */
hyf.calendar =
{
    config: {},  //associative array of field ids to their configuration options.
    activeFieldId: null,
    activeFieldRepeatId: null
};

/**
 * Gets the calendar object for the field with the given ID
 * If this object has already been created it is just returned,
 * otherwise a new calendar object will be created using the configuration options for this field
 * @param fieldId The ID of the field to get the calendar for.  There must be an entry in the
 *                  hyf.calendar.config array for this field
 * @return The CalendarPopup object, or null if one couldn't be created.
 * @private
 * @author Hyfinity Limited
 */
hyf.calendar.getCalendar = function(fieldId)
{
    if (typeof(hyf.calendar.config[fieldId]) == 'undefined')
    {
        return null;
    }
    else if (typeof(hyf.calendar.config[fieldId].calendar) != 'undefined')
    {
        return hyf.calendar.config[fieldId].calendar;
    }
    else
    {
        var options = hyf.calendar.config[fieldId];
        var cal;
        if (options.popup == false)
            cal = new CalendarPopup('calendarDiv');
        else
            cal = new CalendarPopup();

        switch (options.type)
        {
            case 'year_select'  :   cal.showYearNavigation();
                                    break;
            case 'year_entry'   :   cal.showYearNavigation();
                                    cal.showYearNavigationInput();
                                    break;
            case 'navigation_dropdowns' :   cal.showNavigationDropdowns();
                                    cal.setYearSelectStartOffset(100);
                                    break;
        }

        if (typeof(options.minExclusive) != 'undefined')
        {
            cal.addDisabledDates(null, convertDate(options.minExclusive, options.dataFormat, 'y-M-d'));
        }
        if (typeof(options.maxExclusive) != 'undefined')
        {
            cal.addDisabledDates(convertDate(options.maxExclusive, options.dataFormat, 'y-M-d'), null);
        }

        cal.setReturnFunction("hyf.calendar.handleCalendarSelection");

        hyf.calendar.config[fieldId].calendar = cal;
        return hyf.calendar.config[fieldId].calendar;
    }

}

/**
 * Shows the calendar popup for the given field ID.
 * @param fieldId {Field_Name} The ID of the field to show the calendar for.
 * @param repeatId {Repeat_Name} (optional) The ID of the field to show the calendar for within a repeat.
 *
 * @author Hyfinity Limited
 */
hyf.calendar.showCalendar = function(fieldId, repeatId)
{
    if (typeof(repeatId) == 'undefined')
        repeatId = '';

    if ((repeatId == '') && (hyf.calendar.getCalendar(fieldId) == null))
    {
        var names = hyf.util.splitFullFieldId(fieldId);
        repeatId = names.repeatId;
        fieldId = names.fieldName;
    }

    var cal = hyf.calendar.getCalendar(fieldId);
    if ((cal != null) && (typeof(cal) != 'undefined'))
    {
        if (hyf.calendar.config[fieldId].hasDropDowns)
        {
            var format = hyf.validation.DateValidator.getConcatDateFieldParts(repeatId + fieldId, "format", true);
            var value = hyf.validation.DateValidator.getConcatDateFieldParts(repeatId + fieldId, "values", true);
            var preset = '';
            if ((format != '') && (value != ''))
            {
                preset = convertDate(value, format, 'y-M-d');
            }
            cal.showCalendar(repeatId + fieldId + '_calendar_anchor', preset);
        }
        else
        {
            var field = document.getElementById(repeatId + fieldId);
            var preset = ''
            if (field.value != '')
            {
                var format = field.getAttribute('_display_date_format');
                if (format == null)
                    format = field.getAttribute('_data_date_format');
                preset = convertDate(field.value, format, 'y-M-d');
            }
            cal.showCalendar(repeatId + fieldId + '_calendar_anchor', preset);
        }
    }
    hyf.calendar.activeFieldId = fieldId;
    hyf.calendar.activeFieldRepeatId = repeatId;
}

/**
 * Hides the calendar popup for the given field ID.
 * @param fieldId {Field_Name} The ID of the field to hide the calendar for.
 *
 * @author Hyfinity Limited
 */
hyf.calendar.hideCalendar = function(fieldId)
{
    var cal = hyf.calendar.getCalendar(fieldId);

    if (cal == null)
    {
        cal = hyf.calendar.getCalendar(hyf.util.splitFullFieldId(fieldId).fieldName);
    }

    if (cal != null)
        cal.hideCalendar();
}

/**
 * Handles selection of a value from the active calendar popup.
 * This stores the value in the relevant controls on screen.
 * @param y The year value of the selected date
 * @param m The month value of the selected date
 * @param d The day value of the selected date
 * @private
 * @author Hyfinity Limited
 */
hyf.calendar.handleCalendarSelection = function(y, m, d)
{
    if (hyf.calendar.config[hyf.calendar.activeFieldId].hasDropDowns)
    {
        hyf.calendar.setDropdownDateValuesGeneric(y, m, d, hyf.calendar.activeFieldRepeatId + hyf.calendar.activeFieldId);
    }
    else
    {
        var field = document.getElementById(hyf.calendar.activeFieldRepeatId + hyf.calendar.activeFieldId);
        var format = field.getAttribute('_display_date_format');
        if (format == null)
            format = field.getAttribute('_data_date_format');

        field.value = formatDate(new Date(y,m-1,d,0,0,0), format);

        hyf.fireEvent(field, 'change');
    }
}

/**
 * Function that is used when there is a dropdown date format used in conjunction with a pop-up calendar script
 * @private
 * @author Hyfinity Limited
 */
hyf.calendar.setDropdownDateValuesGeneric = function(y, m, d, id)
{
    missCounter = 0;
    partCounter = 1;
    while(missCounter<2)
    {
        var currentField = document.getElementById(id+"_datefield_part_"+partCounter);
        if(currentField == null)
        {
            missCounter++;
        }
        else
        {
            var updated = false;
            if (currentField.type == "select-one")
            {
                if  ((currentField.getAttribute('_display_date_format') == "M") ||
                     (currentField.getAttribute('_display_date_format') == "MM") ||
                     (currentField.getAttribute('_display_date_format') == "MMM") ||
                     (currentField.getAttribute('_display_date_format') == "MMMM"))
                {
                    currentField.selectedIndex = m;
                    updated = true;
                }
                else if ((currentField.getAttribute('_display_date_format') == "d") ||
                         (currentField.getAttribute('_display_date_format') == "dd"))
                {
                    currentField.selectedIndex = d;
                    updated = true;
                }
            }

            if (!updated)
            {
                convertedValue = convertDate(y+"-"+m+"-"+d,"y-M-d",currentField.getAttribute('_display_date_format'));
                currentField.value=convertedValue;
            }

            hyf.fireEvent(currentField, 'change');

            missCounter = 0;
        }
        partCounter++;
    }
}

/*
 * Stores a list of the keywords that are supported for the setDateConstraint function below.
 * For each one a script fragment is provided that will adjust the 'dateToAdjust' date object as needed.
 * @author Hyfinity Limited
 */
hyf.calendar.supportedKeywords = {
    'Today'     : '',
    'Yesterday' : 'dateToAdjust.setTime(dateToAdjust.getTime() - 86400000)',
    'Tomorrow'  : 'dateToAdjust.setTime(dateToAdjust.getTime() + 86400000)',
    'LastWeek'  : 'dateToAdjust.setTime(dateToAdjust.getTime() - 604800000)',
    'NextWeek'  : 'dateToAdjust.setTime(dateToAdjust.getTime() + 604800000)',
    'LastMonth' : 'dateToAdjust.setMonth(dateToAdjust.getMonth() - 1)',
    'NextMonth' : 'dateToAdjust.setMonth(dateToAdjust.getMonth() + 1)',
    'LastYear'  : 'dateToAdjust.setFullYear(dateToAdjust.getFullYear() - 1)',
    'NextYear'  : 'dateToAdjust.setFullYear(dateToAdjust.getFullYear() + 1)'
};


/**
 * This function allows date constraints to be set for a Date Field. This can include resticting the valid date range.
 * Note: to constrain all the field entries in a repeat, use the hyf.calendar.setRepeatDateConstraint function.
 *
 * @param dateField {Field_Name} Set to the Date field to be constrained.
 * @param restrictionType {string} Set to 'Minimum' or 'Maximum' value. Default is 'Minimum'.
 * @param restrictionValue {string | Field_Name} Set to one of the supported keywords, another Date Fieldname, or a data date value. Default is 'Today'.
 * The keyword values are: 'Today', 'Yesterday', 'Tomorrow', 'LastWeek', 'NextWeek', 'LastMonth', 'NextMonth', 'LastYear' or 'NextYear'.
 * @return boolean indicating whether the constraint was successfulyl set up.
 * @author Hyfinity Limited
 */
hyf.calendar.setDateConstraint = function(dateField, restrictionType, restrictionValue, repeatId)
{
    if (typeof(repeatId) == 'undefined')
    {
        var names = hyf.util.splitFullFieldId(dateField);
        repeatId = names.repeatId;
        dateField = names.fieldName;
    }

    var calendarConfig = hyf.calendar.config[dateField];
    var restrictionDateValue = null;
    if ((restrictionType == '') || (typeof(restrictionType) == 'undefined'))
    {
        restrictionType = 'Minimum';
    }
    if ((restrictionValue == '') || (typeof(restrictionValue) == 'undefined'))
    {
        restrictionValue = 'Today';
    }

    if (typeof(hyf.calendar.supportedKeywords[restrictionValue]) != 'undefined')
    {
        var dateToAdjust = new Date();

        if (hyf.calendar.supportedKeywords[restrictionValue] != '')
            eval(hyf.calendar.supportedKeywords[restrictionValue]);

        if (restrictionType == 'Minimum')
        {
            restrictionDateValue = formatDate(new Date(dateToAdjust.getTime() - 86400000), calendarConfig.dataFormat);
        }
        if (restrictionType == 'Maximum')
        {
            restrictionDateValue = formatDate(new Date(dateToAdjust.getTime() + 86400000), calendarConfig.dataFormat);
        }
    }
    else
    {
       var restrictionDate = document.getElementById(restrictionValue);
       if (restrictionDate != null)
       {
           restrictionDateValue = hyf.validation.ValueConverter.performDateConversion(restrictionDate, restrictionDate.value);
       }
       else
       {
           restrictionDateValue = restrictionValue;
       }
    }

    //make sure the restriction value has been set
    if ((restrictionDateValue != null) && (restrictionDateValue != ''))
    {
        var fieldId = repeatId + dateField;
        if (document.getElementById(fieldId))
        {
            var fieldFormat = document.getElementById(fieldId).getAttribute("_data_date_format");

            //make sure the restrictionDateValue actually contains a valid date for this field
            //in some cases the format might be null (eg if called from an onbeforeload event before the attributes are added)
            //so we just allow the constraint if we cant yet confirm the field format.
            if ((fieldFormat == null) || (isDate(restrictionDateValue, fieldFormat)))
            {
                if (restrictionType == 'Minimum')
                {
                    document.getElementById(fieldId).setAttribute('_minExclusive', restrictionDateValue );
                    calendarConfig.minExclusive = restrictionDateValue;
                }
                if (restrictionType == 'Maximum')
                {
                    document.getElementById(fieldId).setAttribute('_maxExclusive', restrictionDateValue );
                    calendarConfig.maxExclusive = restrictionDateValue;
                }
                delete calendarConfig.calendar;
                return true;
            }
        }
    }
    return false;
}


/**
 * This supports setting the appropriate constraint for a date field within a repeat (eg table) structure.
 * This will make sure that the constraint is applied to the date field for each repeat entry (row)
 * @param repeatName {Repeat_Name} The name of the repeat the date field is contained within.
 * @param fieldName {Field_Name} The name of the date field to set the restriction for.
 * @param restrictionType {string} Set to 'Minimum' or 'Maximum' value. Default is 'Minimum'.
 * @param restrictionValue {string | Field_Name} Set to one of the supported keywords, another Date Fieldname, or a data date value. Default is 'Today'.
 * The keyword values are: 'Today', 'Yesterday', 'Tomorrow', 'LastWeek', 'NextWeek', 'LastMonth', 'NextMonth', 'LastYear' or 'NextYear'.
 * @param restrictionValueFieldInRepeat {boolean} (Optional) If set to true, then the restrictionValue is assumed to be the name of a date field within
 *              the same repeat as the field being restricted, and so the appropriate repeat ID is automatically added so that each
 *              field being restricted is based on the field in the same repeat row.
 *
 * @author Hyfinity Limited
 */
hyf.calendar.setRepeatDateConstraint = function(repeatName, fieldName, restrictionType, restrictionValue, restrictionValueFieldInRepeat)
{

    //field name might actually come in with the repeatId at the start, so check for this, and strip it off
    if (fieldName.indexOf(repeatName) == 0)
        fieldName = hyf.util.splitFullFieldId(fieldName).fieldName;


    //loop through the repeat to set the constraint on each field found
    var count = 1;
    var field = document.getElementById(repeatName + count + fieldName);
    while (field != null)
    {
        if ((typeof(restrictionValueFieldInRepeat) != 'undefined') && (restrictionValueFieldInRepeat == true))
            hyf.calendar.setDateConstraint(fieldName, restrictionType, repeatName + count + restrictionValue, repeatName + count);
        else
            hyf.calendar.setDateConstraint(fieldName, restrictionType, restrictionValue, repeatName + count);
        count++;
        field = document.getElementById(repeatName + count + fieldName);
    }

    //now check for editable table insert rows
    field = document.getElementById(repeatName + 'BlankEntry' + fieldName);
    if (field != null)
    {
        if ((typeof(restrictionValueFieldInRepeat) != 'undefined') && (restrictionValueFieldInRepeat == true))
            hyf.calendar.setDateConstraint(fieldName, restrictionType, repeatName + 'BlankEntry' + restrictionValue, repeatName + 'BlankEntry');
        else
            hyf.calendar.setDateConstraint(fieldName, restrictionType, restrictionValue, repeatName + 'BlankEntry');
    }
}


/**
 * This function allows the Mandatory (required) Data Constraint to be set.
 * @param field {Field_Name} Set to the field to be constrained.
 * @param isMandatory {boolean} Set to true if required, and false if not. The Mandatory Marker will be altered as appropriate.
 *
 * @author Hyfinity Limited
 */
hyf.util.setMandatoryConstraint = function(field, isMandatory)
{
    //Escape if mandatory field(s) not populated
    if ((isMandatory == null) || (typeof(isMandatory) != 'boolean'))
        return;
    if (typeof(field) != 'string')
        field = field.getAttribute('name');
    if (field == null)
        return;

    //As Radio and multi-check controls are multiple inputs, we need to loop through with the name to update all of them.
    var nameMatches = dojo.query('*[name=' + field + ']');

    for (var i = 0; i < nameMatches.length; ++i)
    {
       nameMatches[i].setAttribute('_required',isMandatory);
    }

    //Now we need to set the mandatory Markers appropriately.

    var mandatoryMarkerID = field + '_marker';
    var mandatoryMarker = document.getElementById(mandatoryMarkerID);

    if (mandatoryMarker != null)
    {
        // Mandatory market element already present, so change the details
        if (isMandatory == false)
        {
            mandatoryMarker.parentNode.removeChild(mandatoryMarker);
        }
    }
    else
    {
        // Mandatory marker not present, so create marker appropriately in the right location
        if (isMandatory == true)
        {
            var labelForFieldID = field + '_label';
            var containerForFieldID = field + '_container';
            var messageContainerForFieldID = field + '_message_container';

            var newMarkerSpan = document.createElement('span');
            newMarkerSpan.id = mandatoryMarkerID;
            newMarkerSpan.title = 'Mandatory field marker';
            newMarkerSpan.className = hyf.validation.config.mandatoryMarker.className;
            newMarkerSpan.style.cssText = hyf.validation.config.mandatoryMarker.style;
            newMarkerSpan.innerHTML = hyf.validation.config.mandatoryMarker.content;

            switch (hyf.validation.config.mandatoryMarker.location)
            {
                case 'before_label':
                    var labelForField = dojo.byId(labelForFieldID);
                    if (labelForField != null)
                        labelForField.insertBefore(newMarkerSpan, labelForField.firstChild);
                    break;
                case 'after_label':
                    var labelForField = dojo.byId(labelForFieldID);
                    if (labelForField != null)
                        labelForField.appendChild(newMarkerSpan);
                    break;
                case 'before_control':
                    var fieldContainer = dojo.byId(containerForFieldID);
                    if (fieldContainer != null)
                        fieldContainer.insertBefore(newMarkerSpan, fieldContainer.firstChild);
                    break;
                case 'after_control':
                case 'before_container':
                    var messageContainer = dojo.byId(messageContainerForFieldID);
                    if (messageContainer != null)
                        messageContainer.parentNode.insertBefore(newMarkerSpan, messageContainer);
                    else
                    {
                        var fieldContainer = dojo.byId(containerForFieldID);
                        if (fieldContainer != null)
                            fieldContainer.appendChild(newMarkerSpan);
                    }
                    break;
                case 'after_container':
                    var fieldContainer = dojo.byId(containerForFieldID);
                    if (fieldContainer != null)
                        fieldContainer.appendChild(newMarkerSpan);
                    break;
            }
        }
    }
}

/**
 * This function allows the Number Inclusive Range Minimium and Maximum Data constraints to be set.
 * @param field {Field_Name} Set to the field to be constrained (should be a number type field).
 * @param minInclusive {number} (optional) A negative or positive decimal number e.g. -10.00. If blank, an existing value will be removed. If null then the value won't be changed.
 * @param maxInclusive {number} (optional) A negative or positive decimal number e.g. 99999.99.  If blank, an existing value will be removed. If null then the value won't be changed.
 *
 * @author Hyfinity Limited
 */
hyf.util.setInclusiveRangeConstraint = function(field, minInclusive, maxInclusive)
{
    //Escape if mandatory field(s) not populated
    if (field == null)
        return;
    //Allow Numbers, null or blank, but not anything else.
    if ((isNaN(minInclusive)) || (isNaN(maxInclusive)))
        return;

    //first check for a dojo widget, as the fields are moved
    if ((typeof(dijit) != 'undefined') && (typeof(dijit.byId) == 'function') && (dijit.byId(field) != null))
    {
        if (typeof(field) == 'object')
            field = dijit.byId(field).id;
        if (typeof(field) != 'string')
            return;
        if (minInclusive != null)
            if (minInclusive != '')
                dijit.byId(field).constraints.min = minInclusive;
            else
                dijit.byId(field).constraints.min = -Infinity;
        if (maxInclusive != null)
            if (maxInclusive != '')
                dijit.byId(field).constraints.max = maxInclusive;
            else
                dijit.byId(field).constraints.max = +Infinity;
    }
    else
    {
        if (typeof(field) == 'string')
            field = document.getElementById(field);
        if (field == null)
            return;

        if (minInclusive != null)
            if (minInclusive != '')
                field.setAttribute('_minInclusive',minInclusive);
            else
                field.removeAttribute('_minInclusive');
        if (maxInclusive != null)
            if (maxInclusive != '')
                field.setAttribute('_maxInclusive',maxInclusive);
            else
                field.removeAttribute('_maxInclusive');
    }
}

/**
 * This function allows the Number Exclusive Range Minimium and Maximum Data constraints to be set.
 * @param field {Field_Name} Set to the field to be constrained (should be a number type field).
 * @param minExclusive {number} (optional) A negative or positive decimal number e.g. 0.00. If blank, an existing value will be removed. If null then the value won't be changed.
 * @param maxExclusive {number} (optional) A negative or positive decimal number e.g. 99999.99.  If blank, an existing value will be removed. If null then the value won't be changed.
 *
 * @author Hyfinity Limited
 */
hyf.util.setExclusiveRangeConstraint = function(field, minExclusive, maxExclusive)
{
    //Escape if mandatory field(s) not populated
    if (field == null)
        return;
    //Allow Numbers, null or blank, but not anything else.
    if ((isNaN(minExclusive)) || (isNaN(maxExclusive)))
        return;

    //first check for a dojo widget, as the fields are moved
    if ((typeof(dijit) != 'undefined') && (typeof(dijit.byId) == 'function') && (dijit.byId(field) != null))
    {
        if (typeof(field) == 'object')
            field = dijit.byId(field).id;
        if (typeof(field) != 'string')
            return;
    if (minExclusive != null)
            if (minExclusive != '')
            {
                var fixedMinPlaces = Number(minExclusive).toFixed(dijit.byId(field).constraints.places);
                dijit.byId(field).constraints.min = Number(fixedMinPlaces) - 1.00;
            }
            else
                dijit.byId(field).constraints.min = -Infinity;
        if (maxExclusive != null)
            if (maxExclusive != '')
            {
                var fixedMaxPlaces = Number(maxExclusive).toFixed(dijit.byId(field).constraints.places);
                dijit.byId(field).constraints.max = Number(fixedMaxPlaces) + 1.00;
            }
            else
                dijit.byId(field).constraints.max = +Infinity;
    }
    else
    {
        if (typeof(field) == 'string')
            field = document.getElementById(field);
        if (field == null)
            return;

        if (minExclusive != null)
            if (minExclusive != '')
                field.setAttribute('_minExclusive',minExclusive);
            else
                field.removeAttribute('_minExclusive');
        if (maxExclusive != null)
            if (maxExclusive != '')
                field.setAttribute('_maxExclusive',maxExclusive);
            else
                field.removeAttribute('_maxExclusive');
    }
}

/**
 * This function allows the String Length Range Minimium and Maximum Data constraints to be set.
 * @param field {Field_Name} Set to the field to be constrained (should be a string type field).
 * @param minimumLength {number} (optional) A positive number e.g. 0. If blank, an existing value will be removed. If null then the value won't be changed.
 * @param maximumLength {number} (optional) A positive decimal number e.g. 99.  If blank, an existing value will be removed. If null then the value won't be changed.
 * @param fixedLength   {number} (optional) A positive decimal number e.g. 10.  If blank, an existing value will be removed. If null then the value won't be changed.
 *
 * @author Hyfinity Limited
 */
hyf.util.setStringLengthConstraint = function(field, minimumLength, maximumLength, fixedLength)
{
    //Escape if mandatory field(s) not populated
    if (typeof(field) == 'string')
        field = document.getElementById(field);
    if (field == null)
        return;
    //Allow Numbers, null or blank, but not anything else.
    if ((isNaN(minimumLength)) || (isNaN(maximumLength)) || (isNaN(fixedLength)))
        return;

    if ((minimumLength != null) && (minimumLength != '') && (minimumLength >= 0))
    {
        field.setAttribute('_minLength',minimumLength);
    }
    if ((maximumLength != null) && (maximumLength != '') && (maximumLength >= 0))
    {
        field.setAttribute('_maxLength',maximumLength);
        field.setAttribute('maxLength',maximumLength);
    }
    if ((fixedLength != null) && (fixedLength != '') && (fixedLength >= 0))
    {
        field.setAttribute('_length',fixedLength);
        field.setAttribute('maxLength',fixedLength);
    }

    if (minimumLength = '')
    {
        field.removeAttribute('_minLength');
    }
    if (maximumLength = '')
    {
        field.removeAttribute('_maxLength');
    }
    if ((fixedLength = '') && (maximumLength = ''))
    {
        field.removeAttribute('maxLength');
    }
}

/**
 * This function allows the Field Pattern (Regular Expression) Data constraints to be set.
 * @param field {Field_Name} Set to the field to be constrained (should be a string or number type field).
 * @param pattern {string} A string with a regular expression. If blank, an existing value will be removed.
 *
 * @author Hyfinity Limited
 */
hyf.util.setFieldPatternConstraint = function(field, pattern)
{
    //Escape if mandatory field(s) not populated
    if (typeof(field) == 'string')
        field = document.getElementById(field);
    if ((field == null) || (pattern == null))
        return;
    //Allow only strings.
    if (typeof(pattern) != 'string')
        return;

    if (pattern != '')
        field.setAttribute('_regularExpression',pattern);
    else
        field.removeAttribute('_regularExpression');
}

/**
 * This function allows the focus to be placed on a specific field. This will place the cursor in the relvant field.
 * @param field {Field_Name} Set to the field to be constrained (should be a string or number type field).
 *
 * @author Hyfinity Limited
 */
hyf.util.setFocusOnField = function(field)
{
    var fieldName = field;
    //Escape if mandatory field(s) not populated
    if (typeof(field) == 'string')
        field = document.getElementById(field);

    if (field == null)
    {
        //Check if radio or multi-checkbox button as the id of the first option will be the name + 1.
        field = document.getElementById(fieldName + '1');
        if (field == null)
            return;
        if ((field.getAttribute('type') != 'radio') && (field.getAttribute('type') != 'checkbox'))
            return;
    }
    field.focus();
}


/**
 * This function should be called when loading the page for any select box that
 * is being rendered as a dojo filtering select, and has an initial 'please select'
 * entry defined.
 * This function removes this option form the HTML, and ensures that the dojo placeholder
 * value will be initially displayed as needed.
 * @param field The HTML select object before it has been converted to the dojo widget.
 * @private
 * @author Hyfinity Limited
*/
hyf.util.initFilteringSelect = function(field)
{
    if (field.type == 'select-one')
    {
        //check if the intial 'please select' entry is currently selected
        //if so, we need to make sure the select has a value attribute
        //set to '' so that dojo correctly displays the placeholder value instead
        if (field.value == '')
        {
            field = hyf.util.addHtmlAttribute(field, 'value', '');
        }

        //remove the 'please select' option tag so it cant be selected
        field.removeChild(field.getElementsByTagName('option')[0]);
    }
}

/**
 * This method attempts to add an HTML attribute to the given field,
 * rather than setting a object property.
 * For browsers that correctly support setAttribute, this is used,
 * but for older IE browsers the outerHTML string is updated directly.
 * As a result, the reference to the field object could be outdated,
 * and so the new reference is always returned
 * @param field The object representing the HTML field to add the attribute to
 * @param attribute The name of the attribute to add
 * @param value The value to give the new attribute
 * @return The updated field object
 * @private
 * @author Hyfinity Limited
 */
hyf.util.addHtmlAttribute = function(field, attribute, value)
{
    //IE versions before 8 do not correctly support set attribute,
    //so instead we need to update the HTML string
    if (dojo.isIE < 8)
    {
        var id = field.id;
        var oh = field.outerHTML;
        var insertPoint = oh.indexOf('>');
        oh = oh.substring(0, insertPoint) + ' '+attribute+'="'+value+'"' + oh.substr(insertPoint);
        field.outerHTML = oh;
        field = document.getElementById(id);
    }
    else
    {
        field.setAttribute(attribute, value);
    }
    return field;
}

/**
 * This method attempts to remove an HTML attribute from the given field.
 * For browsers that correctly support removeAttribute, this is used,
 * but for older IE browsers the outerHTML string is updated directly.
 * As a result, the reference to the field object could be outdated,
 * and so the new reference is always returned
 * @param field The object representing the HTML field to remove the attribute from
 * @param attribute The name of the attribute to remove
 * @return The updated field object
 * @private
 * @author Hyfinity Limited
 */
hyf.util.removeHtmlAttribute = function(field, attribute)
{
    //IE versions before 8 do not correctly support set attribute,
    //so instead we need to update the HTML string
    if (dojo.isIE < 8)
    {
        var id = field.id;
        var oh = field.outerHTML;
        var elementEnd = oh.indexOf('>');
        var attLoc = oh.indexOf(attribute + '=');
        if ((attLoc == -1) || (attLoc > elementEnd))
            return field;

        var quote = oh.substr(attLoc + 1, 1)
        if ((quote != '"') && (quote != "'"))
            quote = ' ';

        var attEnd = oh.indexOf(quote, attLoc + attribute.length + 2);
        oh = oh.substring(0, attLoc) + oh.substr(attEnd + 1);
        field.outerHTML = oh;
        if(id)
            field = document.getElementById(id);
    }
    else
    {
        field.removeAttribute(attribute);
    }
    return field;
}


/**
 * Checks the given event to see if it represents a backspace key event
 * that should be disabled to stop the browser going back to the previous page.
 * @return true if the event represents a backspace key that will trigger the previous page,
 *          false otherwise
 * @private
 * @author Hyfinity Limited
 */
hyf.util.checkDisableBackspace = function(e)
{
    if (!e)
        e = window.event;

    var key;

     //return when the key is not backspace key
    if(e) {
        key = e.which? e.which : e.keyCode;
        if(key == null || ( key != 8 && key != 13)){
            return false;
        }
    }
    else
    {
        return false;
    }

    //Code is either 8 or 13
    var tag, type, readOnly = false;
    if (e.srcElement)
    {
        //ie
        tag = e.srcElement.tagName.toUpperCase();
        type = e.srcElement.type;
        readOnly =e.srcElement.readOnly;
        if( type == null){ //Type is null means the mouse focus on a non-form field. Disable backspace button
            return true;
        }
        else
        {
            type = e.srcElement.type.toUpperCase();
        }
    } else {
        //firefox
        tag = e.target.nodeName.toUpperCase();
        type = (e.target.type) ? e.target.type.toUpperCase() : "";
    }

    //DO NOT disable backspace ever if focus is not in an input control
    if (tag == 'INPUT' || type == 'TEXT' || type == 'TEXTAREA')
    {
        if(readOnly == true)
        {
            return true;  //if the field is disabled, disable the back space button
        }

        if(((tag == 'INPUT' && type == 'RADIO') || (tag == 'INPUT' && type == 'CHECKBOX')) && (key == 8))
        {
            return true; //disable the backspace button when radio or checkbox
        }

        if((tag == 'INPUT' && type != 'BUTTON') && (key == 13))
        {
            return true; //disable the enter key to prevent issues on single field forms
        }

        return false;

    }

    //if none of above cases, disable the backspace
    return (key == 8);
}

/**
 * This checks the configuration on load to see if we should prevent the backspace key from going back a page.
 * By default we do prevent this, unless the hyf.validation.config.preventBackspace property is set to false.
 * @private
 * @author Hyfinity Limited
 */
hyf.attachEventHandler(window, 'onload', function(){
        if ((typeof(hyf.validation.config.preventBackspace) == 'undefined') ||
            (hyf.validation.config.preventBackspace == true))
        {
            if(dojo.isFF)
            {
                document.onkeypress = function(e){ return !hyf.util.checkDisableBackspace(e)};
            }
            else //IE or Chrome
            {
                document.onkeydown = function(e){ return !hyf.util.checkDisableBackspace(e)};
            }
        }
});


/**
 * This object contains a number of no-op functions that can be connected to
 * to perform custom functionality at appropriate times.
 * Eg for dojo 1.7+
 *  require(['dojo/topic'], function(topic) {
 *      topic.subscribe('hyf/hooks/contentInserted', function(container){....});
 *  });
 * for older dojo versions
 *  dojo.connect(hyf.hooks, 'contentInserted', myCustomFunction)
 *
 * @private
 * @author Hyfinity Limited
 */
hyf.hooks =
{
    /**
     * This will be called whenever content has been inserted into a given container,
     * This will be called after the onbeforeload events have fired, but
     * before any widgets have been parsed, and before onload events have fired.
     * @param container The HTML container that the content has been inseted into.
     * @private
     * @author Hyfinity Limited
     */
    contentInserted: function(container)
    {
        if (typeof(require) == 'function') //dojo 1.7+
        {
            require(["dojo/topic"], function(topic){
                    topic.publish("hyf/hooks/contentInserted", container);
            });
        }
    },
    /**
     * This will be called whenever a container has been parsed for dojo widgets
     * @param container The HTML container that has been parsed.
     * @private
     * @author Hyfinity Limited
     */
    widgetsParsed: function(container)
    {
        if (typeof(require) == 'function') //dojo 1.7+
        {
            require(["dojo/topic"], function(topic){
                    topic.publish("hyf/hooks/widgetsParsed", container);
            });
        }
    }
}

/** Connected to the dojo parser call below to handle calling our hooks function
 * @private
 * @author Hyfinity Limited
 */
hyf.util.dojoParserHandler = function(rootNode, args)
{
    var root;
    if (!args && rootNode && rootNode.rootNode)
        root = rootNode.rootNode;
    else
        root = rootNode;
    root = root ? dojo.byId(root) : document.body;

    hyf.hooks.widgetsParsed(root);
}

//connect up the widgets parsed hook - for now this only handles dojo widgets
if (typeof(require) == 'function') //dojo 1.7+
{
    require(['dojo/parser', 'dojo/aspect'], function (parser, aspect) {
            aspect.after(parser, 'parse', hyf.util.dojoParserHandler, true);
    });
}
else
{
    dojo.ready(function(){
            dojo.connect(dojo.parser, 'parse', hyf.util.dojoParserHandler);
            //initial onload parse will aready have happened by now, so call the hook manually
            hyf.hooks.widgetsParsed(document.body);
    });
}

/**
 * Adds the given function to be called on form submit.
 * If the form param is not provided, then the main webmaker form will be used.
 * @private
 * @author Hyfinity Limited
 */
hyf.util.addOnFormSubmit = function(func, form)
{
    hyf.attachEventHandler(window, 'onload', function(){

            if (!form)
                form = hyf.FMAction.getFormValidator().getForm();

            if (!form.hyfOnSubmitFuncs)
            {
                form.hyfOnSubmitFuncs = new Array();
                if (typeof(form.onsubmit) == "function")
                    form.hyfOnSubmitFuncs.push(form.onsubmit)

                form.onsubmit = function() {return hyf.util.formOnSubmitHandler(form);}
            }

            form.hyfOnSubmitFuncs.push(func)
        });
}

hyf.util.formOnSubmitHandler = function(form)
{
    if (form.hyfOnSubmitFuncs)
    {
        for (var i = 0; i < form.hyfOnSubmitFuncs.length; ++i)
        {
            var resp = form.hyfOnSubmitFuncs[i]();
            if (resp == false)
                return false;
        }
    }
    return true;
}

/**
 * Returns all the dojo dijit widgets in the given container that match the specified class.
 * @param container The container to look in. If not provided the document body will be used.
 * @param dijitClass The class of widgets to find (e.g. 'dijit.Dialog').  If not provided, all
 *          widgets in the container will be returned.
 * @return An array of the found widgets.
 * @private
 * @author Hyfinity Limited
 */
hyf.util.getDijitWidgets = function(container, dijitClass)
{
    if ((typeof(container) == 'undefined') || (container == null))
        container = document.body;

    var widgets = new Array();

    if ((typeof(dijit) != 'undefined') && (typeof(dijit.findWidgets) == 'function'))
    {
        widgets = dijit.findWidgets(container);

        //if a specific class ahs been supplied, then remove any widgets that dont match
        if ((typeof(dijitClass) != 'undefined') && (dijitClass != null))
        {
            for (var i = 0; i < widgets.length; ++i)
            {
                if (widgets[i].declaredClass.indexOf(dijitClass) == -1)
                {
                    widgets.splice(i, 1);
                    --i;
                }
            }
        }
    }

    return widgets;
}

/**
 * Move dijit.dialog content back into the form on submit so that the data
 * it contains will be submitted to the server.
 * @private
 * @author Hyfinity Limited
 */
hyf.util.addOnFormSubmit(function(){

    var form = hyf.FMAction.getFormValidator().getForm();
    var widgets = hyf.util.getDijitWidgets(document.body, 'dijit.Dialog');
    for (var i = 0; i < widgets.length; ++i)
    {
        var widget = widgets[i];
        if (widget.containerNode)
        {
            //make sure the content remains hidden, and move it back into the main form
            widget.containerNode.style.display = 'none';
            form.appendChild(widget.containerNode);
        }
    }
});


hyf.util.conditionalDisplay =
{
    desc:   'Handles functioanlity for hiding and showing groups or fields as field values change on screen',
    settings: new Array() //stores all the conditional settings objects that have been processed.
}

/**
 * Should be called to setup a new component for conditional display.
 * @private
 * @author Hyfinity Limited
 * @param details An object describing the component, and the conditions for when
 *                it should be displayed. Format:

{
    type: 'field|group',
    name: '' //field or group name
    repeatId: '',   //optional
    condition: comparison|comparisonOperator
}

comparisonOperator = {
    type: 'and|or',
    values: [comparison|comparisonOperator, comparison|comparisonOperator, ...] //array of length >= 2
}

comparison = {
    type: '==|!=|<|>|<=|>=|contains',
    values: ['string value'|comparisonValue, 'string value'|comparisonValue] //array of length = 2
}

comparisonValue = {
    type: 'static|field|displayvariable',
    name: '', //static value or field name or display variable name
    repeatId: '', //optional for fields not needed otherwise
}

 */
hyf.util.conditionalDisplay.setup = function(details)
{
    var id = hyf.util.conditionalDisplay.settings.length;
    hyf.util.conditionalDisplay.settings[id] = details;

    hyf.util.conditionalDisplay.updateDisplay(details);

    return id;
}

require(['dojo/topic'], function(topic) {
        topic.subscribe('hyf/hooks/widgetsParsed', function(container){
                for (var i = 0; i < hyf.util.conditionalDisplay.settings.length; ++i)
                {
                    var details = hyf.util.conditionalDisplay.settings[i];
                    if (!details.initialised)
                    {
                        hyf.util.conditionalDisplay.setupCondition(details.condition, i);
                        details.initialised = true;
                    }
                }
        });
});



/**
 * Processes the given condition to ensure that all contained
 * source fields are correctly initialised.
 * @param cond The condition object to process from the passed in setup details.
 * @param id the ID in the settings collection for this particualr setup.
 * @private
 * @author Hyfinity Limited
  */
hyf.util.conditionalDisplay.setupCondition = function(cond, id)
{
    //find all fields used within the conditions to add events to them where needed
    if ((cond.type == 'and') || (cond.type == 'or'))
    {
        //is a complex condition with multiple checks.
        for (var i = 0; i < cond.values.length; ++i)
        {
            hyf.util.conditionalDisplay.setupCondition(cond.values[i], id);
        }

    }
    else
    {
        //is a simple condition with a single check
        hyf.util.conditionalDisplay.setupSourceField(cond.values[0], id);
        hyf.util.conditionalDisplay.setupSourceField(cond.values[1], id);
    }
}

/**
 * Processes the given condition value to check if it is a field or display variable.
 * If so events are added to the field so that nay chanegs to its value will be detected.
 * @param cv The condition value object to process from the passed in setup details.
 * @param id the ID in the settings collection for this particualr setup.
 * @private
 * @author Hyfinity Limited
 */
hyf.util.conditionalDisplay.setupSourceField = function(cv, id)
{
    if (typeof(cv) == 'string')
        return;

    if (cv.type == 'field')
    {
        var fieldname = cv.name;
        if (cv.repeatId)
            fieldname = cv.repeatId + fieldname;

        var field = null;

        var connectEvent = 'onchange';

        //check for dojo widgets
        if ((typeof(dijit) != 'undefined') && (typeof(dijit.byId) == 'function') && (dijit.byId(fieldname) != null))
        {
            field = dijit.byId(fieldname);
            connectEvent = 'onChange';
        }
        else
        {
            field = document.getElementById(fieldname)
        }

        if (field)
        {
            if (!field._hyfCondDisplayIds)
            {
                if ((field.type == 'checkbox') || (field.type == 'radio'))
                    connectEvent = 'onclick';

                field._hyfCondDisplayIds = new Array();
                hyf.attachEventHandler(field, connectEvent, hyf.util.conditionalDisplay.handleValueChange);
            }

            field._hyfCondDisplayIds.push(id);
        }
        else
        {
            //likely to be a radio or multi check control so need to add events to multiple fields
            var nameMatches = dojo.query('*[name=' + fieldname + ']');
            for (var i = 0; i < nameMatches.length; ++i)
            {
                if (!nameMatches[i]._hyfCondDisplayIds)
                {
                    nameMatches[i]._hyfCondDisplayIds = new Array();
                    hyf.attachEventHandler(nameMatches[i], 'onclick', hyf.util.conditionalDisplay.handleValueChange);
                }

                nameMatches[i]._hyfCondDisplayIds.push(id);
            }
        }

    }
    else if (cv.type == 'displayvariable')
    {
        var field = document.getElementById('hyf_display_variable_' + cv.name);
        if (field)
        {
            if (!field._hyfCondDisplayIds)
            {
                field._hyfCondDisplayIds = new Array();
                hyf.attachEventHandler(field, 'onchange', hyf.util.conditionalDisplay.handleValueChange);
            }

            field._hyfCondDisplayIds.push(id);
        }
    }
}

/**
 * Will be called from a watched field when the value changes.
 * This will recheck all the components conditionally dispalyed based on this field
 * and update the display as needed.
 * @param e The change event triggering the call.
 * @private
 * @author Hyfinity Limited
 */
hyf.util.conditionalDisplay.handleValueChange = function(e)
{
    if (this._hyfCondDisplayIds)
    {
        for (var i = 0; i < this._hyfCondDisplayIds.length; ++i)
        {
            hyf.util.conditionalDisplay.updateDisplay(hyf.util.conditionalDisplay.settings[this._hyfCondDisplayIds[i]]);
        }
    }

}

/**
 * Updates the display of the component on screen so that it is only visible if the
 * specified conditions are true.
 * @param details The conditional setup details object specifying the component and conditions.
 *                See the setup method for the structure.
 * @private
 * @author Hyfinity Limited
 */
hyf.util.conditionalDisplay.updateDisplay = function(details)
{
    var makeVisible = hyf.util.conditionalDisplay.getConditionTruth(details.condition);

    var compName = details.name;
    if (details.repeatId)
        compName = details.repeatId + compName;

    var componentsToAdjust = new Array();

    var fieldContainer, labelContainer;



    if (details.type == 'group')
    {
        var fieldContainer = document.getElementById(compName);

        var groupLabel = document.getElementById(compName + '_label_container');

        if (groupLabel && !hyf.util.isParent(groupLabel, fieldContainer))
            labelContainer = groupLabel;
    }
    else if (details.type == 'field')
    {
        fieldContainer = document.getElementById(compName + '_container');
        labelContainer = document.getElementById(compName + '_label_container');
    }


    if (fieldContainer == null)  //shouldn't ever happen, but if it does there's not a lot we can do
        return;

    //if there is a formElement container, then just hide that
    if (dojo.hasClass(fieldContainer.parentNode, 'formElement'))
        componentsToAdjust.push(fieldContainer.parentNode);
    else
    {
        //otherwise look for a field and label to hide both containers

        var hideContents = false;


        //need to handle tables though - just hide the contents rather than the cell??
        if (fieldContainer.tagName.toLowerCase() == 'td')
        {
            var table = fieldContainer.parentNode;
            var tr;
            while(table && table.tagName.toLowerCase() != 'table')
            {
                if (table.tagName.toLowerCase() == 'tr')
                    tr = table;
                table = table.parentNode;
            }

            if (table)
            {
                //if this is a grid, then we cant hide the tds as it
                //would affect the layout
                if (dojo.hasClass(table, 'grid'))
                    hideContents = true;

                //if this is a repeating table then check if any of the condition fields
                //are within the same repeat context.
                //If so, then also dont hide the cell itself as otherwise the remaining cells
                //would not line up with the column headings
                //If all the conditions are outside of the repeat, then we do want to hide the cell,
                //so that the whole column gets removed.
                if (dojo.hasClass(tr, 'table'))
                {
                    if ((typeof(details.repeatId) != 'undefined') && (details.repeatId != ''))
                    {
                        var conditionsToCheck = new Array();
                        conditionsToCheck.push(details.condition);

                        var item;
                        while ((item = conditionsToCheck.shift()) != null)
                        {
                            if (item.type == 'or' || item.type == 'and')
                            {
                                conditionsToCheck = conditionsToCheck.concat(item.values);
                            }
                            else
                            {
                                if ((item.values[0].repeatId && item.values[0].repeatId.indexOf(details.repeatId) == 0) ||
                                    (item.values[1].repeatId && item.values[1].repeatId.indexOf(details.repeatId) == 0))
                                {
                                    //condition is in same repeat, so just hide this one cell.
                                    hideContents = true;
                                    break;
                                }
                            }
                        }
                        if (!hideContents)
                        {
                            //not hiding the contents so want to hide the whole cell
                            //in this case, also want to make sure the column heading label is hidden
                            //as well.  This will not have the repeat Id component on the name
                            labelContainer = document.getElementById(details.name + '_label_container');
                        }
                    }
                    else
                        hideContents = true;
                }
            }

        }


        if (hideContents)
        {
            //find all children to hide instead
            for (var i = 0; i < fieldContainer.childNodes.length; ++i)
            {
                componentsToAdjust.push(fieldContainer.childNodes[i]);
            }

            if (labelContainer)
            {
                for (var i = 0; i < labelContainer.childNodes.length; ++i)
                {
                    componentsToAdjust.push(labelContainer.childNodes[i]);
                }
            }
        }
        else
        {
            componentsToAdjust.push(fieldContainer);
            if (labelContainer)
                componentsToAdjust.push(labelContainer);
        }


    }

    if (makeVisible)
    {
        for (var i = 0; i < componentsToAdjust.length; ++i)
        {
            hyf.util.showComponent(componentsToAdjust[i]);
        }
    }
    else
    {
        for (var i = 0; i < componentsToAdjust.length; ++i)
        {
            hyf.util.hideComponent(componentsToAdjust[i]);
        }
    }

}

/**
 * Checks whether the given condition object currently evalautes to a true value.
 * This will process all the checks in the condition and return the final result.
 * @param cond The condition object from the setup details structure to process.
 * @return boolean result.
 * @private
 * @author Hyfinity Limited
 */
hyf.util.conditionalDisplay.getConditionTruth = function(cond)
{

    if (cond.type == 'and')
    {
        //all nested conditions must be true
        for (var i = 0; i < cond.values.length; ++i)
        {
            if (!hyf.util.conditionalDisplay.getConditionTruth(cond.values[i]))
                return false;
        }
        return true;
    }
    else if (cond.type == 'or')
    {
        //one nested condition must be true
        var result = false;
        for (var i = 0; i < cond.values.length; ++i)
        {
            if (hyf.util.conditionalDisplay.getConditionTruth(cond.values[i]))
            {
                result = true;
                break;
            }
        }
        return result;
    }
    else
    {
        //is a simple condition with a single check

        //Need to handle static values that could have multiple strings separated by ;
        //so the value propertyies of these will always actually contain an array of values.
        var value1 = hyf.util.conditionalDisplay.getConditionValue(cond.values[0]);
        var value2 = hyf.util.conditionalDisplay.getConditionValue(cond.values[1]);

        var dataType;
        if (value1.field)
            dataType = value1.field.getAttribute('_type');
        if (!dataType && value2.field)
            dataType = value2.field.getAttribute('_type');

        if (dataType == 'date')
        {
            if (value1.field)
            {
                value1.value = [hyf.validation.ValueConverter.performDateConversion(value1.field, value1.value[0])];
                value1.format = value1.field.getAttribute('_data_date_format');
            }

            if (value2.field)
            {
                value2.value = [hyf.validation.ValueConverter.performDateConversion(value2.field, value2.value[0])];
                value2.format = value2.field.getAttribute('_data_date_format');
            }
        }

        for (var i = 0; i < value1.value.length; ++i)
        {
            for (var j = 0; j < value2.value.length; ++j)
            {

                if (hyf.util.compareValues(value1.value[i], value2.value[j], cond.type, dataType, value1.format, value2.format))
                    return true;

            }
        }

        return false;
    }
}

/**
 * Gets the actual value for the given condition value object.
 * This will get the values of the fields on screen where needed.
 * @param cv The condition value object form the conditional setup details.
 * @return An object containing a value property and optional field property
 *         the value will always be an array even though in most cases it will have just a single entry.
 * @private
 * @author Hyfinity Limited
 */
hyf.util.conditionalDisplay.getConditionValue = function(cv)
{
    if (typeof(cv) == 'string')
    {
        var val;
        //check for multiple values separated by ;
        if (cv.indexOf(';') != -1)
        {
            val = cv.split(';');
        }
        else
        {
            val = [cv];
        }

        return {value: val};
    }
    else if (cv.type == 'field')
    {
        var fieldname = cv.name;
        if (cv.repeatId)
            fieldname = cv.repeatId + fieldname;

        //getFieldValue will return a string, or an array if multiple values selected.
        var val = hyf.util.getFieldValue(fieldname);
        if (!((typeof(val) == 'object') && (val.length)))
            val = [val];

        return {value: val, field: document.getElementById(fieldname)};
    }
    else if (cv.type == 'displayvariable')
    {
        return {value: [hyf.util.getDisplayVariableValue(cv.name)]};
    }
}


/**
 * Compares two values and returns the true or false result
 * @param val1 The first value to compare
 * @param val2 The second value to compare
 * @param comparison the type of comparison. Must be one of '==', '!=', '<', '>', '<=', '>=', 'contains'
 * @param dataType (Optional) the data type of the values. Supported options 'string', 'number', 'date'
 * @param val1Format (Optional) If the dataType is data, this should be used to set the format of the val1 date
 * @param val2Format (Optional) If the dataType is data, this should be used to set the format of the val2 date
 *                   If both dates are in the same format this is not needed.
 * @return boolean value indicating the result of the comparison.
 * @private
 * @author Hyfinity Limited
 */
hyf.util.compareValues = function(val1, val2, comparison, dataType, val1Format, val2Format)
{
    try
    {
        comparison = comparison.replace('&lt;', '<').replace('&gt;', '>');

        if (dataType == 'date')
        {
            //make sure we have at least one format string
            if (!val1Format && !val2Format)
                return false;

            //check for the defined date keywords
            if ((val1.charAt(0) == '$') && (hyf.calendar.supportedKeywords[val1.substr(1)] != null))
            {
                var dateToAdjust = new Date();

                if (hyf.calendar.supportedKeywords[val1.substr(1)] != '')
                    eval(hyf.calendar.supportedKeywords[val1.substr(1)]);

                val1 = formatDate(dateToAdjust, ((val1Format) ? val1Format : val2Format));
            }

            if ((val2.charAt(0) == '$') && (hyf.calendar.supportedKeywords[val2.substr(1)] != null))
            {
                var dateToAdjust = new Date();

                if (hyf.calendar.supportedKeywords[val2.substr(1)] != '')
                    eval(hyf.calendar.supportedKeywords[val2.substr(1)]);

                val2 = formatDate(dateToAdjust, ((val2Format) ? val2Format : val1Format));
            }

            var compareResult;
            if (val1Format && val2Format)
            {
                compareResult = compareDates(val1, val1Format, val2, val2Format);
            }
            else
            {
                var f = (val1Format) ? val1Format : val2Format;
                compareResult = compareDates(val1, f, val2, f);
            }

            //compareResult will be -1, 0, 1
            if (compareResult == 'error')
                return false;

            switch (comparison)
            {
                case '=='   :   return (compareResult == 0);
                case '!='   :   return (compareResult != 0);
                case '<'    :   return (compareResult == -1);
                case '>'    :   return (compareResult == 1);
                case '<='   :   return (compareResult < 1);
                case '>='   :   return (compareResult > -1);
                case 'contains' : return false;
            }

            //invalid comparison
            return false;
        }
        else
        {
            if (comparison == 'contains')
                return (val1.indexOf(val2) != -1)
            else
            {
                //check if both values are numbers, and if so try and do numeric comparison
                //We need to have the isNaN check as for display variable checks we dont have a dataType
                //However if we have a string dataType then we should treat as strings even if they are both numbers
                try
                {
                    if ((dataType == 'number') || ((dataType != 'string') && !isNaN(val1) && !isNaN(val2)))
                    {
                        return eval('Number(val1) ' + comparison + ' Number(val2)');
                    }
                }
                catch (e) {}

                return eval('val1 ' + comparison + ' val2');
            }
        }
    }
    catch (e)
    {
        //if anything has gone wrong in the comparison, then just return false.
        return false;
    }

}
